@@ -13,9 +13,7 @@
 # along with Gajim. If not, see <http://www.gnu.org/licenses/>.
 
 from typing import Any
-from typing import Optional
 from typing import TextIO
-from typing import Union
 
 import cProfile
 import json
@@ -60,8 +58,6 @@ from gajim.common.storage.draft import DraftStorage
 from gajim.common.storage.events import EventStorage
 from gajim.common.task_manager import TaskManager
 from gajim.common.util.http import create_http_request
-from gajim.plugins.pluginmanager import PluginManager
-from gajim.plugins.repository import PluginRepository
 
 
 class CoreApplication(ged.EventHelper):
@@ -122,6 +118,9 @@ class CoreApplication(ged.EventHelper):
             app.nicks[account] = app.settings.get_account_setting(account,
                                                                   'name')
 
+        from gajim.plugins.pluginmanager import PluginManager
+        from gajim.plugins.repository import PluginRepository
+
         app.plugin_manager = PluginManager()
         app.plugin_manager.init_plugins()
         app.plugin_repository = PluginRepository()
@@ -212,7 +211,7 @@ class CoreApplication(ged.EventHelper):
         self._profiling_session.disable()
         self._log.info('End profiling')
         ps = pstats.Stats(self._profiling_session)
-        ps = ps.sort_stats(SortKey.CUMULATIVE)
+        ps = ps.sort_stats(SortKey.TIME)
         ps.print_stats()
 
     def start_shutdown(self, *args: Any, **kwargs: Any) -> None:
@@ -259,12 +258,12 @@ class CoreApplication(ged.EventHelper):
         import warnings
         os.environ['GAJIM_LEAK'] = 'true'
 
-        def warn_with_traceback(message: Union[Warning, str],
+        def warn_with_traceback(message: Warning | str,
                                 category: type[Warning],
                                 filename: str,
                                 lineno: int,
-                                _file: Optional[TextIO] = None,
-                                line: Optional[str] = None) -> None:
+                                _file: TextIO | None = None,
+                                line: str | None = None) -> None:
 
             traceback.print_stack(file=sys.stderr)
             sys.stderr.write(warnings.formatwarning(message, category,
@@ -314,10 +313,10 @@ class CoreApplication(ged.EventHelper):
                      callback=self._on_update_response)
 
     def _on_update_response(self, request: HTTPRequest) -> None:
-
         if not request.is_complete():
-            self._log.warning('Error while performing gajim update check: %s',
-                              request.get_error(), request.get_status())
+            self._log.warning(
+                'Error while performing gajim update check: %s (%s)',
+                request.get_error_string(), request.get_status())
             return
 
         now = datetime.now()
@@ -338,7 +337,7 @@ class CoreApplication(ged.EventHelper):
 
     def create_account(self,
                        account: str,
-                       username: str,
+                       username: str | None,
                        domain: str,
                        password: str,
                        proxy_name: str | None,
@@ -348,12 +347,15 @@ class CoreApplication(ged.EventHelper):
                        anonymous: bool = False
                        ) -> None:
 
-        account_label = f'{username}@{domain}'
         if anonymous:
             username = 'anon'
-            account_label = f'anon@{domain}'
 
-        config: dict[str, Union[str, int, bool]] = {
+        if not username:
+            raise ValueError('Username must be set')
+
+        account_label = f'{username}@{domain}'
+
+        config: dict[str, str | int | bool] = {
             'name': username,
             'resource': f'gajim.{get_random_string(8)}',
             'account_label': account_label,
@@ -377,8 +379,9 @@ class CoreApplication(ged.EventHelper):
         for opt, value in config.items():
             app.settings.set_account_setting(account, opt, value)  # pyright: ignore  # noqa: E501
 
-        # Password module depends on existing config
-        passwords.save_password(account, password)
+        if not anonymous:
+            # Password module depends on existing config
+            passwords.save_password(account, password)
 
     def enable_account(self, account: str) -> None:
         app.connections[account] = Client(account)
@@ -428,7 +431,7 @@ class CoreApplication(ged.EventHelper):
 
     def change_status(self,
                       status: str,
-                      account: Optional[str] = None
+                      account: str | None = None
                       ) -> None:
 
         if account is not None:
