@@ -15,7 +15,6 @@
 from __future__ import annotations
 
 from typing import Any
-from typing import Optional
 
 import logging
 import time
@@ -34,6 +33,7 @@ from gajim.common import app
 from gajim.common import helpers
 from gajim.common import modules
 from gajim.common import passwords
+from gajim.common.client_modules import ClientModules
 from gajim.common.const import ClientState
 from gajim.common.const import SimpleClientState
 from gajim.common.events import AccountConnected
@@ -63,12 +63,13 @@ from gajim.gtk.util import open_window
 log = logging.getLogger('gajim.client')
 
 
-IgnoredTlsErrorsT = Optional[set[Gio.TlsCertificateFlags]]
+IgnoredTlsErrorsT = set[Gio.TlsCertificateFlags] | None
 
 
-class Client(Observable):
+class Client(Observable, ClientModules):
     def __init__(self, account: str) -> None:
         Observable.__init__(self, log)
+        ClientModules.__init__(self, account)
         self._client = None
         self._account = account
         self.name = account
@@ -144,7 +145,7 @@ class Client(Observable):
         return self._client.features
 
     @property
-    def local_address(self) -> Optional[str]:
+    def local_address(self) -> str | None:
         address = self._client.local_address
         if address is not None:
             return address.to_string().split(':')[0]
@@ -243,6 +244,7 @@ class Client(Observable):
                         account=self._account,
                         client=self,
                         cert=cert,
+                        ignored_errors=set(self._client.ignored_tls_errors),
                         error=errors.pop())
 
         elif domain in (StreamError.STREAM, StreamError.BIND):
@@ -264,7 +266,7 @@ class Client(Observable):
 
             if error in ('not-authorized', 'no-password'):
                 def _on_password() -> None:
-                    self._prepare_for_connect()
+                    self.connect()
 
                 app.ged.raise_event(PasswordRequired(client=self,
                                                      on_password=_on_password))
@@ -375,7 +377,7 @@ class Client(Observable):
             if show == 'offline':
                 return
 
-            self._prepare_for_connect()
+            self.connect()
             return
 
         if self._state.is_connecting:
@@ -390,7 +392,7 @@ class Client(Observable):
                 self._destroy_client = True
                 self._abort_reconnect()
             else:
-                self._prepare_for_connect()
+                self.connect()
             return
 
         # We are connected
@@ -422,9 +424,6 @@ class Client(Observable):
         if include_muc:
             self.get_module('MUC').update_presence()
 
-    def get_module(self, name: str):
-        return modules.get(self._account, name)
-
     @helpers.call_counter
     def connect_machine(self) -> None:
         log.info('Connect machine state: %s', self._connect_machine_calls)
@@ -522,7 +521,18 @@ class Client(Observable):
                         msg_log_id=log_line_id,
                         play_sound=message.play_sound))
 
-    def _prepare_for_connect(self) -> None:
+    def connect(
+        self,
+        ignored_tls_errors: IgnoredTlsErrorsT = None
+    ) -> None:
+
+        log.info('Connect')
+
+        if self._state not in (ClientState.DISCONNECTED,
+                               ClientState.RECONNECT_SCHEDULED):
+            # Do not try to reco while we are already trying
+            return
+
         custom_host = get_custom_host(self._account)
         if custom_host is not None:
             self._client.set_custom_host(*custom_host)
@@ -548,19 +558,10 @@ class Client(Observable):
         password = passwords.get_password(self._account)
         self._client.set_password(password)
 
-        self.connect()
-
-    def connect(self, ignored_tls_errors: IgnoredTlsErrorsT = None) -> None:
-        if self._state not in (ClientState.DISCONNECTED,
-                               ClientState.RECONNECT_SCHEDULED):
-            # Do not try to reco while we are already trying
-            return
-
-        log.info('Connect')
-
         self._client.set_accepted_certificates(
             app.cert_store.get_certificates())
         self._client.set_ignored_tls_errors(ignored_tls_errors)
+
         self._reconnect = True
         self._disable_reconnect_timer()
         self._set_state(ClientState.CONNECTING)
@@ -579,7 +580,7 @@ class Client(Observable):
         self._set_state(ClientState.RECONNECT_SCHEDULED)
         log.info('Reconnect to %s in 3s', self._account)
         self._reconnect_timer_source = GLib.timeout_add_seconds(
-            3, self._prepare_for_connect)
+            3, self.connect)
 
     def _abort_reconnect(self) -> None:
         self._set_state(ClientState.DISCONNECTED)
