@@ -19,38 +19,41 @@
 # You should have received a copy of the GNU General Public License
 # along with Gajim. If not, see <http://www.gnu.org/licenses/>.
 
-from typing import Dict  # pylint: disable=unused-import
-from typing import List
-from typing import Generator
-from typing import Optional  # pylint: disable=unused-import
-from typing import Tuple
+from __future__ import annotations
 
+from typing import cast
+
+import importlib.resources
 import os
 import sys
 import tempfile
+from collections.abc import Generator
 from pathlib import Path
 
 from gi.repository import GLib
 
 import gajim
+from gajim.common.const import PathLocation
+from gajim.common.const import PathType
 from gajim.common.i18n import _
-from gajim.common.const import PathType, PathLocation
-from gajim.common.types import PathTuple
+
+PathTupleT = tuple[PathLocation | None, Path, PathType | None]
 
 
-def get(key: str) -> str:
+def get(key: str) -> Path:
     return _paths[key]
 
 
-def get_plugin_dirs() -> List[str]:
+def get_plugin_dirs() -> list[Path]:
     if gajim.IS_FLATPAK:
-        return ['/app/plugins',
-                _paths['PLUGINS_BASE']]
-    return [_paths['PLUGINS_BASE'],
-            _paths['PLUGINS_USER']]
+        return [Path(_paths['PLUGINS_BASE']),
+                Path('/app/plugins')]
+    return [Path(_paths['PLUGINS_BASE']),
+            Path(_paths['PLUGINS_USER'])]
 
 
-def get_paths(type_: PathType) -> Generator[str, None, None]:
+def get_paths(type_: PathType) -> Generator[Path, None, None]:
+    # pylint: disable=unnecessary-dict-index-lookup
     for key, value in _paths.items():
         path_type = value[2]
         if type_ != path_type:
@@ -58,10 +61,6 @@ def get_paths(type_: PathType) -> Generator[str, None, None]:
         yield _paths[key]
 
 
-def override_path(*args, **kwargs):
-    _paths.add(*args, **kwargs)
-
-
 def set_separation(active: bool) -> None:
     _paths.profile_separation = active
 
@@ -71,7 +70,7 @@ def set_profile(profile: str) -> None:
 
 
 def set_config_root(config_root: str) -> None:
-    _paths.custom_config_root = str(Path(config_root).resolve())
+    _paths.custom_config_root = Path(config_root).resolve()
 
 
 def init() -> None:
@@ -80,92 +79,89 @@ def init() -> None:
 
 def create_paths() -> None:
     for path in get_paths(PathType.FOLDER):
-        if not isinstance(path, Path):
-            path_ = Path(path)
-
-        if path_.is_file():
-            print(_('%s is a file but it should be a directory') % path_)
+        if path.is_file():
+            print(_('%s is a file but it should be a directory') % path)
             print(_('Gajim will now exit'))
             sys.exit()
 
-        if not path_.exists():
-            for parent_path in reversed(path_.parents):
+        if not path.exists():
+            for parent_path in reversed(path.parents):
                 # Create all parent folders
                 # don't use mkdir(parent=True), as it ignores `mode`
                 # when creating the parents
                 if not parent_path.exists():
                     print(('creating %s directory') % parent_path)
                     parent_path.mkdir(mode=0o700)
-            print(('creating %s directory') % path_)
-            path_.mkdir(mode=0o700)
+            print(('creating %s directory') % path)
+            path.mkdir(mode=0o700)
 
 
 class ConfigPaths:
     def __init__(self) -> None:
-        self._paths = {}  # type: Dict[str, PathTuple]
+        self._paths: dict[str, PathTupleT] = {}
         self.profile = ''
         self.profile_separation = False
-        self.custom_config_root = None  # type: Optional[str]
+        self.custom_config_root: Path | None = None
 
         if os.name == 'nt':
-            try:
+            if gajim.IS_PORTABLE:
+                application_path = Path(sys.executable).parent
+                self.config_root = self.cache_root = self.data_root = \
+                    application_path.parent / 'UserData'
+            else:
                 # Documents and Settings\[User Name]\Application Data\Gajim
                 self.config_root = self.cache_root = self.data_root = \
-                        os.path.join(os.environ['appdata'], 'Gajim')
-            except KeyError:
-                # win9x, in cwd
-                self.config_root = self.cache_root = self.data_root = '.'
+                    Path(os.environ['APPDATA']) / 'Gajim'
         else:
-            self.config_root = os.path.join(GLib.get_user_config_dir(),
-                                            'gajim')
-            self.cache_root = os.path.join(GLib.get_user_cache_dir(), 'gajim')
-            self.data_root = os.path.join(GLib.get_user_data_dir(), 'gajim')
+            self.config_root = Path(GLib.get_user_config_dir()) / 'gajim'
+            self.cache_root = Path(GLib.get_user_cache_dir()) / 'gajim'
+            self.data_root = Path(GLib.get_user_data_dir()) / 'gajim'
 
-        import pkg_resources
-        basedir = pkg_resources.resource_filename("gajim", ".")
+        basedir = cast(Path, importlib.resources.files('gajim'))
 
         source_paths = [
-            ('DATA', os.path.join(basedir, 'data')),
-            ('STYLE', os.path.join(basedir, 'data', 'style')),
-            ('EMOTICONS', os.path.join(basedir, 'data', 'emoticons')),
-            ('GUI', os.path.join(basedir, 'data', 'gui')),
-            ('ICONS', os.path.join(basedir, 'data', 'icons')),
-            ('HOME', os.path.expanduser('~')),
-            ('PLUGINS_BASE', os.path.join(basedir, 'data', 'plugins')),
+            ('DATA', basedir / 'data'),
+            ('STYLE', basedir / 'data' / 'style'),
+            ('GUI', basedir / 'data' / 'gui'),
+            ('ICONS', basedir / 'data' / 'icons'),
+            ('HOME', Path.home()),
+            ('PLUGINS_BASE', basedir / 'data' / 'plugins'),
         ]
 
         for path in source_paths:
             self.add(*path)
 
-    def __getitem__(self, key: str) -> str:
+    def __getitem__(self, key: str) -> Path:
         location, path, _ = self._paths[key]
         if location == PathLocation.CONFIG:
-            return os.path.join(self.config_root, path)
+            return self.config_root / path
         if location == PathLocation.CACHE:
-            return os.path.join(self.cache_root, path)
+            return self.cache_root / path
         if location == PathLocation.DATA:
-            return os.path.join(self.data_root, path)
+            return self.data_root / path
         return path
 
-    def items(self) -> Generator[Tuple[str, PathTuple], None, None]:
-        for key, value in self._paths.items():
-            yield (key, value)
+    def items(self) -> Generator[tuple[str, PathTupleT], None, None]:
+        yield from self._paths.items()
 
-    def _prepare(self, path: str, unique: bool) -> str:
+    def _prepare(self, path: Path, unique: bool) -> Path:
         if os.name == 'nt':
-            path = path.capitalize()
+            path = Path(str(path).capitalize())
         if self.profile:
             if unique or self.profile_separation:
-                return '%s.%s' % (path, self.profile)
+                return Path(f'{path}.{self.profile}')
         return path
 
     def add(self,
             name: str,
-            path: str,
-            location: PathLocation = None,
-            path_type: PathType = None,
+            path: Path | str,
+            location: PathLocation | None = None,
+            path_type: PathType | None = None,
             unique: bool = False) -> None:
-        if path and location is not None:
+
+        path = Path(path)
+
+        if location is not None:
             path = self._prepare(path, unique)
         self._paths[name] = (location, path, path_type)
 
@@ -175,10 +171,10 @@ class ConfigPaths:
             self.cache_root = self.data_root = self.custom_config_root
 
         user_dir_paths = [
-            ('TMP', tempfile.gettempdir()),
-            ('MY_CONFIG', '', PathLocation.CONFIG, PathType.FOLDER),
-            ('MY_CACHE', '', PathLocation.CACHE, PathType.FOLDER),
-            ('MY_DATA', '', PathLocation.DATA, PathType.FOLDER),
+            ('TMP', Path(tempfile.gettempdir()), None, None),
+            ('MY_CONFIG', Path(), PathLocation.CONFIG, PathType.FOLDER),
+            ('MY_CACHE', Path(), PathLocation.CACHE, PathType.FOLDER),
+            ('MY_DATA', Path(), PathLocation.DATA, PathType.FOLDER),
         ]
 
         for path in user_dir_paths:
@@ -188,16 +184,18 @@ class ConfigPaths:
         unique_profile_paths = [
             # Data paths
             ('SECRETS_FILE', 'secrets', PathLocation.DATA, PathType.FILE),
-            ('MY_PEER_CERTS', 'certs', PathLocation.DATA, PathType.FOLDER),
             ('CERT_STORE', 'cert_store', PathLocation.DATA, PathType.FOLDER),
             ('DEBUG', 'debug', PathLocation.DATA, PathType.FOLDER),
-            ('PLUGINS_DATA', 'plugins_data', PathLocation.DATA, PathType.FOLDER),
+            ('PLUGINS_DATA', 'plugins_data',
+             PathLocation.DATA, PathType.FOLDER),
 
             # Config paths
+            ('SETTINGS', 'settings.sqlite', PathLocation.CONFIG, PathType.FILE),
             ('CONFIG_FILE', 'config', PathLocation.CONFIG, PathType.FILE),
             ('PLUGINS_CONFIG_DIR',
              'pluginsconfig', PathLocation.CONFIG, PathType.FOLDER),
-            ('MY_CERT', 'localcerts', PathLocation.CONFIG, PathType.FOLDER),
+            ('MY_SHORTCUTS', 'shortcuts.json',
+             PathLocation.CONFIG, PathType.FILE),
         ]
 
         for path in unique_profile_paths:
@@ -208,11 +206,11 @@ class ConfigPaths:
         paths = [
             # Data paths
             ('LOG_DB', 'logs.db', PathLocation.DATA, PathType.FILE),
-            ('MY_CACERTS', 'cacerts.pem', PathLocation.DATA, PathType.FILE),
-            ('PLUGINS_DOWNLOAD', 'plugins_download', PathLocation.CACHE, PathType.FOLDER),
+            ('PLUGINS_DOWNLOAD', 'plugins_download',
+             PathLocation.CACHE, PathType.FOLDER),
+            ('PLUGINS_IMAGES', 'plugins_images',
+             PathLocation.CACHE, PathType.FOLDER),
             ('PLUGINS_USER', 'plugins', PathLocation.DATA, PathType.FOLDER),
-            ('MY_EMOTS',
-             'emoticons', PathLocation.DATA, PathType.FOLDER_OPTIONAL),
             ('MY_ICONSETS',
              'iconsets', PathLocation.DATA, PathType.FOLDER_OPTIONAL),
 
