@@ -22,7 +22,6 @@
 from __future__ import annotations
 
 from typing import cast
-from typing import Optional
 
 import ctypes
 import ctypes.util
@@ -284,7 +283,7 @@ class IdleMonitorManager(GObject.Object):
         self._idle_monitor = self._get_idle_monitor()
 
         if self.is_available():
-            GLib.timeout_add_seconds(1, self._poll)
+            GLib.timeout_add_seconds(5, self._poll)
 
     def set_interval(self,
                      away_interval: int = 60,
@@ -323,7 +322,7 @@ class IdleMonitorManager(GObject.Object):
         return self.state == IdleState.UNKNOWN
 
     @staticmethod
-    def _get_idle_monitor() -> Optional[IdleMonitor]:
+    def _get_idle_monitor() -> IdleMonitor | None:
         if sys.platform == 'win32':
             return Windows()
 
