@@ -23,6 +23,7 @@ from pathlib import Path
 
 from nbxmpp.structs import StanzaHandler
 
+from gajim.common import app
 from gajim.common.modules.base import BaseModule
 
 if typing.TYPE_CHECKING:
@@ -39,7 +40,7 @@ _store_publish_modules = [
 
 
 def register_modules(client: Client) -> None:
-    if client in _modules:
+    if client.account in _modules:
         return
 
     _modules[client.account] = {}
@@ -80,6 +81,7 @@ def unregister_modules(client: Client) -> None:
     for instance in _modules[client.account].values():
         if hasattr(instance, 'cleanup'):
             instance.cleanup()
+        app.check_finalize(instance)
     del _modules[client.account]
 
 
