@@ -13,26 +13,23 @@
 # along with Gajim. If not, see <http://www.gnu.org/licenses/>.
 
 from typing import Any
-from typing import Callable
+from typing import cast
 from typing import Literal
 from typing import NamedTuple
-from typing import Optional
-from typing import Dict
-from typing import List
-from typing import Union
 from typing import overload
-from typing import cast
+from typing import TypedDict
 
-import sys
+import inspect
 import json
-import uuid
 import logging
 import sqlite3
-import inspect
+import sys
+import uuid
 import weakref
-from pathlib import Path
-from collections import namedtuple
 from collections import defaultdict
+from collections import namedtuple
+from collections.abc import Callable
+from pathlib import Path
 
 from gi.repository import GLib
 from nbxmpp.protocol import JID
@@ -41,47 +38,49 @@ from gajim import IS_PORTABLE
 from gajim.common import app
 from gajim.common import configpaths
 from gajim.common import optparser
-from gajim.common.storage.base import Encoder
-from gajim.common.storage.base import json_decoder
-from gajim.common.setting_values import APP_SETTINGS
+from gajim.common.setting_values import ACCOUNT_SETTINGS
 from gajim.common.setting_values import AllContactSettings
 from gajim.common.setting_values import AllContactSettingsT
-from gajim.common.setting_values import BoolContactSettings
-from gajim.common.setting_values import StringContactSettings
 from gajim.common.setting_values import AllGroupChatSettings
 from gajim.common.setting_values import AllGroupChatSettingsT
-from gajim.common.setting_values import BoolGroupChatSettings
-from gajim.common.setting_values import IntGroupChatSettings
-from gajim.common.setting_values import StringGroupChatSettings
+from gajim.common.setting_values import AllSettingsT
 from gajim.common.setting_values import AllWorkspaceSettings
 from gajim.common.setting_values import AllWorkspaceSettingsT
-from gajim.common.setting_values import OpenChatsSettingT
-from gajim.common.setting_values import StringWorkspaceSettings
-from gajim.common.setting_values import ACCOUNT_SETTINGS
-from gajim.common.setting_values import PROXY_SETTINGS
-from gajim.common.setting_values import PROXY_EXAMPLES
-from gajim.common.setting_values import INITAL_WORKSPACE
-from gajim.common.setting_values import PLUGIN_SETTINGS
-from gajim.common.setting_values import WORKSPACE_SETTINGS
-from gajim.common.setting_values import DEFAULT_SOUNDEVENT_SETTINGS
-from gajim.common.setting_values import STATUS_PRESET_SETTINGS
-from gajim.common.setting_values import STATUS_PRESET_EXAMPLES
-from gajim.common.setting_values import HAS_APP_DEFAULT
-from gajim.common.setting_values import HAS_ACCOUNT_DEFAULT
-from gajim.common.setting_values import AllSettingsT
-from gajim.common.setting_values import IntSettings
-from gajim.common.setting_values import IntAccountSettings
-from gajim.common.setting_values import StringSettings
-from gajim.common.setting_values import StringAccountSettings
-from gajim.common.setting_values import BoolSettings
+from gajim.common.setting_values import APP_SETTINGS
 from gajim.common.setting_values import BoolAccountSettings
+from gajim.common.setting_values import BoolContactSettings
+from gajim.common.setting_values import BoolGroupChatSettings
+from gajim.common.setting_values import BoolSettings
+from gajim.common.setting_values import DEFAULT_SOUNDEVENT_SETTINGS
+from gajim.common.setting_values import FloatSettings
+from gajim.common.setting_values import HAS_ACCOUNT_DEFAULT
+from gajim.common.setting_values import HAS_APP_DEFAULT
+from gajim.common.setting_values import INITAL_WORKSPACE
+from gajim.common.setting_values import IntAccountSettings
+from gajim.common.setting_values import IntGroupChatSettings
+from gajim.common.setting_values import IntSettings
+from gajim.common.setting_values import OpenChatsSettingT
+from gajim.common.setting_values import PLUGIN_SETTINGS
+from gajim.common.setting_values import PROXY_EXAMPLES
+from gajim.common.setting_values import PROXY_SETTINGS
+from gajim.common.setting_values import STATUS_PRESET_EXAMPLES
+from gajim.common.setting_values import STATUS_PRESET_SETTINGS
+from gajim.common.setting_values import StringAccountSettings
+from gajim.common.setting_values import StringContactSettings
+from gajim.common.setting_values import StringGroupChatSettings
+from gajim.common.setting_values import StringSettings
+from gajim.common.setting_values import StringWorkspaceSettings
+from gajim.common.setting_values import WORKSPACE_SETTINGS
+from gajim.common.setting_values import WorkspaceSettings
+from gajim.common.storage.base import Encoder
+from gajim.common.storage.base import json_decoder
 
-SETTING_TYPE = Union[bool, int, str, object]
+SETTING_TYPE = bool | int | str | object
 
 
 log = logging.getLogger('gajim.c.settings')
 
-CURRENT_USER_VERSION = 2
+CURRENT_USER_VERSION = 5
 
 CREATE_SQL = '''
     CREATE TABLE settings (
@@ -94,46 +93,58 @@ CREATE_SQL = '''
             settings TEXT
     );
 
-    INSERT INTO settings(name, settings) VALUES ('app', '{}');
-    INSERT INTO settings(name, settings) VALUES ('soundevents', '{}');
-    INSERT INTO settings(name, settings) VALUES ('status_presets', '%s');
-    INSERT INTO settings(name, settings) VALUES ('proxies', '%s');
-    INSERT INTO settings(name, settings) VALUES ('plugins', '{}');
-    INSERT INTO settings(name, settings) VALUES ('workspaces', '%s');
+    INSERT INTO settings(name, settings) VALUES ('app', '{{}}');
+    INSERT INTO settings(name, settings) VALUES ('soundevents', '{{}}');
+    INSERT INTO settings(name, settings) VALUES ('status_presets', '{status}');
+    INSERT INTO settings(name, settings) VALUES ('proxies', '{proxies}');
+    INSERT INTO settings(name, settings) VALUES ('plugins', '{{}}');
+    INSERT INTO settings(name, settings) VALUES ('workspaces', '{workspaces}');
 
-    PRAGMA user_version=%s;
-    ''' % (json.dumps(STATUS_PRESET_EXAMPLES),
-           json.dumps(PROXY_EXAMPLES),
-           json.dumps(INITAL_WORKSPACE),
-           CURRENT_USER_VERSION)
+    PRAGMA user_version={version};
+    '''.format(status=json.dumps(STATUS_PRESET_EXAMPLES),
+               proxies=json.dumps(PROXY_EXAMPLES),
+               workspaces=json.dumps(INITAL_WORKSPACE),
+               version=CURRENT_USER_VERSION)
 
 
-_SignalCallable = Callable[[Any, str, Optional[str], Optional[JID]], Any]
-_CallbackDict = dict[tuple[str, Optional[str], Optional[JID]],
+_SignalCallable = Callable[[Any, str, str | None, JID | None], Any]
+_CallbackDict = dict[tuple[str, str | None, JID | None],
                      list[weakref.WeakMethod[_SignalCallable]]]
 
 if app.is_flatpak():
-    OVERRIDES_PATH = Path('/app/app-overrides.json')
+    app_overrides = '/app/app-overrides.json'
 else:
-    OVERRIDES_PATH = Path('/etc/gajim/app-overrides.json')
+    app_overrides = '/etc/gajim/app-overrides.json'
+OVERRIDES_PATH = Path(app_overrides)
+
+
+class SettingsDictT(TypedDict):
+    app: dict[str, Any]
+    plugins: dict[str, dict[str, Any]]
+    workspaces: dict[str, dict[str, WorkspaceSettings]]
+    soundevents: dict[str, dict[str, Any]]
+    status_presets: dict[str, dict[str, str]]
+    proxies: dict[str, dict[str, Any]]
 
 
 class Settings:
-    def __init__(self):
+    def __init__(self, in_memory: bool = False):
         self._con = cast(sqlite3.Connection, None)
         self._commit_scheduled = None
+        self._in_memory = in_memory
 
-        self._settings = {}
+        self._settings: SettingsDictT = {}
         self._app_overrides: dict[str, AllSettingsT] = {}
-        self._account_settings = {}
+        self._account_settings: dict[
+            str, Any | dict[str, dict[JID | str, Any]]] = {}
 
         self._callbacks: _CallbackDict = defaultdict(list)
 
     def connect_signal(self,
                        setting: str,
                        func: _SignalCallable,
-                       account: Optional[str] = None,
-                       jid: Optional[JID] = None) -> None:
+                       account: str | None = None,
+                       jid: JID | None = None) -> None:
         if not inspect.ismethod(func):
             # static methods are not bound to an object so we can’t easily
             # remove the func once it should not be called anymore
@@ -143,7 +154,7 @@ class Settings:
         self._callbacks[(setting, account, jid)].append(weak_func)
 
     def disconnect_signals(self, object_: object) -> Any:
-        for _, handlers in self._callbacks.items():
+        for handlers in self._callbacks.values():
             for handler in list(handlers):
                 if isinstance(handler, tuple):
                     continue
@@ -155,10 +166,10 @@ class Settings:
                     setting: str,
                     widget: Any,
                     func_name: str,
-                    account: Optional[str] = None,
-                    jid: Optional[JID] = None,
+                    account: str | None = None,
+                    jid: JID | None = None,
                     inverted: bool = False,
-                    default_text: Optional[str] = None
+                    default_text: str | None = None
                     ) -> None:
 
         callbacks = self._callbacks[(setting, account, jid)]
@@ -173,8 +184,8 @@ class Settings:
     def _notify(self,
                 value: Any,
                 setting: str,
-                account: Optional[str] = None,
-                jid: Optional[JID] = None) -> None:
+                account: str | None = None,
+                jid: JID | None = None) -> None:
 
         log.info('Signal: %s changed', setting)
 
@@ -209,7 +220,10 @@ class Settings:
 
     def init(self) -> None:
         self._setup_installation_defaults()
-        self._connect_database()
+        if self._in_memory:
+            self._connect_in_memory_database()
+        else:
+            self._connect_database()
         self._load_settings()
         self._load_account_settings()
         if not self._settings['app']:
@@ -226,10 +240,8 @@ class Settings:
         if IS_PORTABLE:
             APP_SETTINGS['use_keyring'] = False
 
-        if sys.platform in ('win32', 'darwin'):
-            # Cairo does not support color emojis fonts on
-            # Windows and MacOS
-            APP_SETTINGS['dev_use_message_label'] = False
+        if sys.platform == 'win32':
+            APP_SETTINGS['app_font_size'] = 1.125
 
     def _load_app_overrides(self) -> None:
         if not OVERRIDES_PATH.exists():
@@ -248,7 +260,7 @@ class Settings:
     def _namedtuple_factory(cursor: sqlite3.Cursor,
                             row: tuple[Any, ...]) -> NamedTuple:
         fields = [col[0] for col in cursor.description]
-        return namedtuple('Row', fields)(*row)  # type: ignore
+        return namedtuple('Row', fields)(*row)  # pyright: ignore
 
     def _connect_database(self) -> None:
         path = configpaths.get('SETTINGS')
@@ -262,6 +274,20 @@ class Settings:
         self._con = sqlite3.connect(path)
         self._con.row_factory = self._namedtuple_factory
 
+    def _connect_in_memory_database(self) -> None:
+        log.info('Creating in memory')
+        self._con = sqlite3.connect(':memory:')
+        self._con.row_factory = self._namedtuple_factory
+
+        try:
+            self._con.executescript(CREATE_SQL)
+        except Exception:
+            log.exception('Error')
+            self._con.close()
+            sys.exit()
+
+        self._con.commit()
+
     @staticmethod
     def _create_database(statement: str, path: Path) -> None:
         log.info('Creating %s', path)
@@ -307,6 +333,9 @@ class Settings:
         self._con.commit()
 
     def _migrate_database(self) -> None:
+        if self._in_memory:
+            return
+
         try:
             self._migrate()
         except Exception:
@@ -326,7 +355,7 @@ class Settings:
         if version < 2:
             # Migrate open chats to new key and format
             for workspace in self._settings['workspaces'].values():
-                open_chats = []
+                open_chats: list[dict[str, Any]] = []
                 for open_chat in workspace.get('open_chats', []):
                     account, jid, type_, pinned = open_chat
                     open_chats.append({'account': account,
@@ -340,7 +369,33 @@ class Settings:
 
             self._set_user_version(2)
 
+        if version < 3:
+            # Migrate open chats to new key and format
+            for account_settings in self._account_settings.values():
+                if account_settings['account'].get('active') is None:
+                    account_settings['account']['active'] = True
+
+            self._set_user_version(3)
+
+        if version < 4:
+            value = self._settings['app'].get('chat_timestamp_format')
+            if value is not None:
+                self._settings['app']['time_format'] = value
+
+            value = self._settings['app'].get('date_timestamp_format')
+            if value is not None:
+                self._settings['app']['date_format'] = value
+
+            self._set_user_version(4)
+
+        if version < 5:
+            self._settings['app'].pop('muclumbus_api_http_uri', None)
+            self._set_user_version(5)
+
     def _migrate_old_config(self) -> None:
+        if self._in_memory:
+            return
+
         config_file = configpaths.get('CONFIG_FILE')
         if not config_file.exists():
             return
@@ -389,12 +444,12 @@ class Settings:
         # Migrate deprecated settings
         value = app_settings.pop('send_chatstate_muc_default', None)
         if value is not None:
-            for account, settings in self._account_settings.items():
+            for settings in self._account_settings.values():
                 settings['account']['gc_send_chatstate_default'] = value
 
         value = app_settings.pop('send_chatstate_default', None)
         if value is not None:
-            for account, settings in self._account_settings.items():
+            for settings in self._account_settings.values():
                 settings['account']['send_chatstate_default'] = value
 
         value = app_settings.pop('print_join_left_default', None)
@@ -425,6 +480,7 @@ class Settings:
             if account is None:
                 continue
 
+            assert jid is not None
             encryption = settings.get('encryption')
             if not encryption:
                 continue
@@ -450,8 +506,8 @@ class Settings:
                     jid]['encryption'] = encryption
             self._commit_account_settings(account)
 
-    def _split_encryption_config_key(self, key: str) -> tuple[Optional[str],
-                                                              Optional[str]]:
+    def _split_encryption_config_key(self, key: str) -> tuple[str | None,
+                                                              str | None]:
         for account in self._account_settings:
             if not key.startswith(account):
                 continue
@@ -576,6 +632,10 @@ class Settings:
     def get_app_setting(self, setting: IntSettings) -> int:
         ...
 
+    @overload
+    def get_app_setting(self, setting: FloatSettings) -> float:
+        ...
+
     def get_app_setting(self, setting: str) -> AllSettingsT:
         if setting not in APP_SETTINGS:
             raise ValueError(f'Invalid app setting: {setting}')
@@ -590,19 +650,25 @@ class Settings:
     @overload
     def set_app_setting(self,
                         setting: BoolSettings,
-                        value: Optional[bool]) -> None:
+                        value: bool | None) -> None:
         ...
 
     @overload
     def set_app_setting(self,
                         setting: StringSettings,
-                        value: Optional[str]) -> None:
+                        value: str | None) -> None:
         ...
 
     @overload
     def set_app_setting(self,
                         setting: IntSettings,
-                        value: Optional[int]) -> None:
+                        value: int | None) -> None:
+        ...
+
+    @overload
+    def set_app_setting(self,
+                        setting: FloatSettings,
+                        value: float | None) -> None:
         ...
 
     @overload
@@ -613,7 +679,7 @@ class Settings:
 
     def set_app_setting(self,
                         setting: str,
-                        value: Optional[AllSettingsT]) -> None:
+                        value: AllSettingsT | None) -> None:
 
         if setting not in APP_SETTINGS:
             raise ValueError(f'Invalid app setting: {setting}')
@@ -657,7 +723,7 @@ class Settings:
         except KeyError:
             return PLUGIN_SETTINGS[setting]
 
-    def get_plugins(self) -> List[str]:
+    def get_plugins(self) -> list[str]:
         return list(self._settings['plugins'].keys())
 
     def set_plugin_setting(self,
@@ -688,6 +754,9 @@ class Settings:
 
     def add_account(self, account: str) -> None:
         log.info('Add account: %s', account)
+        if account in self._account_settings:
+            raise ValueError('Account %s exists already' % account)
+
         self._account_settings[account] = {'account': {},
                                            'contact': {},
                                            'group_chat': {}}
@@ -706,11 +775,19 @@ class Settings:
             (account,))
         self._commit()
 
-    def get_accounts(self) -> List[str]:
+    def get_accounts(self) -> list[str]:
         return list(self._account_settings.keys())
 
-    def get_active_accounts(self) -> List[str]:
-        active = []
+    def account_exists(self, jid: str) -> bool:
+        for account in self._account_settings:
+            name = self.get_account_setting(account, 'name')
+            hostname = self.get_account_setting(account, 'hostname')
+            if jid == f'{name}@{hostname}':
+                return True
+        return False
+
+    def get_active_accounts(self) -> list[str]:
+        active: list[str] = []
         for account in self._account_settings:
             if self.get_account_setting(account, 'active'):
                 active.append(account)
@@ -753,27 +830,27 @@ class Settings:
     def set_account_setting(self,
                             account: str,
                             setting: StringAccountSettings,
-                            value: Optional[str]) -> None:
+                            value: str | None) -> None:
         ...
 
     @overload
     def set_account_setting(self,
                             account: str,
                             setting: IntAccountSettings,
-                            value: Optional[int]) -> None:
+                            value: int | None) -> None:
         ...
 
     @overload
     def set_account_setting(self,
                             account: str,
                             setting: BoolAccountSettings,
-                            value: Optional[bool]) -> None:
+                            value: bool | None) -> None:
         ...
 
     def set_account_setting(self,
                             account: str,
                             setting: str,
-                            value: Optional[AllSettingsT]) -> None:
+                            value: AllSettingsT | None) -> None:
 
         if account not in self._account_settings:
             raise ValueError(f'Account missing: {account}')
@@ -872,7 +949,7 @@ class Settings:
                                account: str,
                                jid: JID,
                                setting: StringGroupChatSettings,
-                               value: str) -> None:
+                               value: str | None) -> None:
         ...
 
     @overload
@@ -880,7 +957,7 @@ class Settings:
                                account: str,
                                jid: JID,
                                setting: IntGroupChatSettings,
-                               value: int) -> None:
+                               value: int | None) -> None:
         ...
 
     @overload
@@ -888,14 +965,14 @@ class Settings:
                                account: str,
                                jid: JID,
                                setting: BoolGroupChatSettings,
-                               value: bool) -> None:
+                               value: bool | None) -> None:
         ...
 
     def set_group_chat_setting(self,
                                account: str,
                                jid: JID,
                                setting: AllGroupChatSettings,
-                               value: AllGroupChatSettingsT) -> None:
+                               value: AllGroupChatSettingsT | None) -> None:
 
         if account not in self._account_settings:
             raise ValueError(f'Account missing: {account}')
@@ -905,15 +982,16 @@ class Settings:
 
         default = ACCOUNT_SETTINGS['group_chat'][setting]
         if default in (HAS_APP_DEFAULT, HAS_ACCOUNT_DEFAULT):
-
-            client = app.get_client(account)
-            contact = client.get_module('Contacts').get_contact(jid)
-            context = contact.muc_context
-            if context is None:
-                # If there is no disco info available
-                # to determine the context assume public
-                log.warning('Unable to determine context for: %s', jid)
-                context = 'public'
+            context = 'public'
+            if app.account_is_connected(account):
+                client = app.get_client(account)
+                contact = client.get_module('Contacts').get_contact(jid)
+                context = contact.muc_context
+                if context is None:
+                    # If there is no disco info available
+                    # to determine the context assume public
+                    log.warning('Unable to determine context for: %s', jid)
+                    context = 'public'
 
             default_store = APP_SETTINGS
             if default is HAS_ACCOUNT_DEFAULT:
@@ -949,7 +1027,8 @@ class Settings:
     def set_group_chat_settings(self,
                                 setting: str,
                                 value: SETTING_TYPE,
-                                context: str = None) -> None:
+                                context: str | None = None
+                                ) -> None:
 
         for account, acc_settings in self._account_settings.items():
             for jid in acc_settings['group_chat']:
@@ -1005,7 +1084,7 @@ class Settings:
                             account: str,
                             jid: JID,
                             setting: StringContactSettings,
-                            value: str) -> None:
+                            value: str | None) -> None:
         ...
 
     @overload
@@ -1013,14 +1092,14 @@ class Settings:
                             account: str,
                             jid: JID,
                             setting: BoolContactSettings,
-                            value: bool) -> None:
+                            value: bool | None) -> None:
         ...
 
     def set_contact_setting(self,
                             account: str,
                             jid: JID,
                             setting: AllContactSettings,
-                            value: AllContactSettingsT) -> None:
+                            value: AllContactSettingsT | None) -> None:
 
         if account not in self._account_settings:
             raise ValueError(f'Account missing: {account}')
@@ -1092,7 +1171,9 @@ class Settings:
         self._commit_settings('soundevents')
 
     def get_soundevent_settings(self,
-                                event_name: str) -> Dict[str, SETTING_TYPE]:
+                                event_name: str
+                                ) -> dict[str, SETTING_TYPE]:
+
         if event_name not in DEFAULT_SOUNDEVENT_SETTINGS:
             raise ValueError(f'Invalid soundevent: {event_name}')
 
@@ -1121,7 +1202,7 @@ class Settings:
 
         self._commit_settings('status_presets')
 
-    def get_status_preset_settings(self, status_preset: str) -> Dict[str, str]:
+    def get_status_preset_settings(self, status_preset: str) -> dict[str, str]:
         if status_preset not in self._settings['status_presets']:
             raise ValueError(f'Invalid status preset name: {status_preset}')
 
@@ -1130,7 +1211,7 @@ class Settings:
         settings.update(user_settings)
         return settings
 
-    def get_status_presets(self) -> List[str]:
+    def get_status_presets(self) -> list[str]:
         return list(self._settings['status_presets'].keys())
 
     def remove_status_preset(self, status_preset: str) -> None:
@@ -1160,7 +1241,7 @@ class Settings:
 
         self._commit_settings('proxies')
 
-    def get_proxy_settings(self, proxy_name: str) -> Dict[str, SETTING_TYPE]:
+    def get_proxy_settings(self, proxy_name: str) -> dict[str, SETTING_TYPE]:
         if proxy_name not in self._settings['proxies']:
             raise ValueError(f'Unknown proxy: {proxy_name}')
 
@@ -1169,7 +1250,7 @@ class Settings:
         settings.update(user_settings)
         return settings
 
-    def get_proxies(self) -> List[str]:
+    def get_proxies(self) -> list[str]:
         return list(self._settings['proxies'].keys())
 
     def add_proxy(self, proxy_name: str) -> None:
@@ -1259,10 +1340,13 @@ class Settings:
         except KeyError:
             return WORKSPACE_SETTINGS[setting]
 
-    def get_workspaces(self) -> List[str]:
+    def get_workspace_count(self) -> int:
+        return len(self._settings['workspaces'].keys())
+
+    def get_workspaces(self) -> list[str]:
         workspace_order = app.settings.get_app_setting('workspace_order')
 
-        def sort_workspaces(workspace_id):
+        def sort_workspaces(workspace_id: str) -> int:
             try:
                 return workspace_order.index(workspace_id)
             except ValueError:
@@ -1286,6 +1370,15 @@ class Settings:
         del self._settings['workspaces'][id_]
         self._commit_settings('workspaces')
 
+    def shutdown(self) -> None:
+        if self._commit_scheduled is not None:
+            GLib.source_remove(self._commit_scheduled)
+            self._commit_scheduled = None
+
+        self._commit()
+        self._con.close()
+        del self._con
+
 
 class LegacyConfig:
 
