@@ -16,7 +16,6 @@ from __future__ import annotations
 
 from typing import Any
 from typing import cast
-from typing import Optional
 
 import locale
 import logging
@@ -34,7 +33,7 @@ from gajim.common import types
 from gajim.common.const import ClientState
 from gajim.common.i18n import _
 from gajim.common.i18n import p_
-from gajim.common.settings import AllSettingsT
+from gajim.common.setting_values import AllSettingsT
 
 from gajim.gtk.const import Setting
 from gajim.gtk.const import SettingKind
@@ -169,7 +168,7 @@ class AccountsWindow(Gtk.ApplicationWindow):
         if not initial:
             self._accounts[account].show()
 
-    def select_account(self, account: str, page: Optional[str] = None) -> None:
+    def select_account(self, account: str, page: str | None = None) -> None:
         try:
             self._accounts[account].select(page)
         except KeyError:
@@ -466,7 +465,7 @@ class Account:
         self._account_row = AccountRow(account)
         self._menu.add_account(self._account_row)
 
-    def select(self, page_name: Optional[str] = None) -> None:
+    def select(self, page_name: str | None = None) -> None:
         self._account_row.emit('activate')
         if page_name is not None:
             self._menu.set_page(self._account, page_name)
@@ -621,13 +620,10 @@ class AddNewAccountPage(Gtk.Box):
         self.set_vexpand(True)
         self.set_hexpand(True)
         self.set_margin_top(24)
-        pixbuf = Gtk.IconTheme.load_icon_for_scale(
-            Gtk.IconTheme.get_default(),
-            'org.gajim.Gajim-symbolic',
-            100,
-            self.get_scale_factor(),
-            Gtk.IconLookupFlags.FORCE_SIZE)
-        self.add(Gtk.Image.new_from_pixbuf(pixbuf))
+        image = Gtk.Image.new_from_icon_name(
+            'org.gajim.Gajim-symbolic', Gtk.IconSize.from_name('100'))
+        image.set_opacity(0.2)
+        self.add(image)
 
         button = Gtk.Button(label=_('Add Account'))
         button.get_style_context().add_class('suggested-action')
@@ -763,7 +759,7 @@ class PrivacyPage(GenericSettingPage):
 
     def __init__(self, account: str) -> None:
         self._account = account
-        self._client: Optional[types.Client] = None
+        self._client: types.Client | None = None
         if app.account_is_connected(account):
             self._client = app.get_client(account)
 
@@ -779,12 +775,27 @@ class PrivacyPage(GenericSettingPage):
         }
 
         chatstate_entries = {
-            'all': _('Enabled'),
-            'composing_only': _('Composing Only'),
             'disabled': _('Disabled'),
+            'composing_only': _('Composing Only'),
+            'all': _('All Chat States'),
+        }
+
+        encryption_entries = {
+            '': _('Unencrypted'),
+            'OMEMO': 'OMEMO',
+            'OpenPGP': 'OpenPGP',
+            'PGP': 'PGP',
         }
 
         settings = [
+            Setting(SettingKind.POPOVER,
+                    _('Default Encryption'),
+                    SettingType.ACCOUNT_CONFIG,
+                    'encryption_default',
+                    desc=_('Encryption method to use '
+                           'unless overridden on a per-contact basis'),
+                    props={'entries': encryption_entries}),
+
             Setting(SettingKind.SWITCH,
                     _('Idle Time'),
                     SettingType.ACCOUNT_CONFIG,
@@ -942,9 +953,9 @@ class EncryptionOMEMOPage(GenericSettingPage):
 
         btbv_label = Gtk.Label()
         btbv_label.set_xalign(0)
-        markup = '<a href="%s">%s</a>' % (
-            'https://dev.gajim.org/gajim/gajim/-/wikis/help/OMEMO',
-            _('Read more about blind trust'))
+        wiki_url = 'https://dev.gajim.org/gajim/gajim/-/wikis/help/OMEMO'
+        link_text = _('Read more about blind trust')
+        markup = f'<a href="{wiki_url}">{link_text}</a>'
         btbv_label.set_markup(markup)
         self.add(btbv_label)
 
@@ -1057,9 +1068,9 @@ class PriorityDialog(SettingsDialog):
 
         neg_priority = app.settings.get('enable_negative_priority')
         if neg_priority:
-            range_ = (-128, 127)
+            range_ = (-128, 127, 1)
         else:
-            range_ = (0, 127)
+            range_ = (0, 127, 1)
 
         settings = [
             Setting(SettingKind.SWITCH,
@@ -1113,7 +1124,7 @@ class CutstomHostnameDialog(SettingsDialog):
                     SettingType.ACCOUNT_CONFIG,
                     'custom_port',
                     bind='account::use_custom_host',
-                    props={'range_': (0, 65535)},),
+                    props={'range_': (0, 65535, 1)}),
 
             Setting(SettingKind.COMBO,
                     _('Type'),
