@@ -14,13 +14,17 @@
 
 from typing import cast
 
+import logging
+
 from cryptography.hazmat.primitives import hashes
 from cryptography.hazmat.primitives.asymmetric.dsa import DSAPublicKey
 from cryptography.hazmat.primitives.asymmetric.ec import EllipticCurvePublicKey
 from cryptography.hazmat.primitives.asymmetric.ed448 import Ed448PublicKey
 from cryptography.hazmat.primitives.asymmetric.ed25519 import Ed25519PublicKey
 from cryptography.hazmat.primitives.asymmetric.rsa import RSAPublicKey
+from cryptography.utils import int_to_bytes
 from cryptography.x509 import DNSName
+from cryptography.x509 import ExtensionNotFound
 from cryptography.x509.oid import ExtensionOID
 from gi.repository import Gdk
 from gi.repository import Gio
@@ -29,9 +33,12 @@ from gi.repository import Gtk
 from gajim.common import app
 from gajim.common.helpers import get_x509_cert_from_gio_cert
 from gajim.common.i18n import _
+from gajim.common.util.text import format_bytes_as_hex
 
 from gajim.gtk.builder import get_builder
 
+log = logging.getLogger('gajim.gtk.certificate_dialog')
+
 
 class CertificateDialog(Gtk.ApplicationWindow):
     def __init__(self,
@@ -82,18 +89,22 @@ class CertificateBox(Gtk.Box):
                 self._it_organization = str(attribute.value)
 
         # Get the subjectAltName extension from the certificate
-        subject_ext = cert.extensions.get_extension_for_oid(
-            ExtensionOID.SUBJECT_ALTERNATIVE_NAME)
-        # Get the DNSName entries from the SAN extension
-        alt_names = cast(
-            list[str],
-            subject_ext.value.get_values_for_type(DNSName))  # pyright: ignore
-        self._it_subject_alt_names = '\n'.join(alt_names)
+        try:
+            subject_ext = cert.extensions.get_extension_for_oid(
+                ExtensionOID.SUBJECT_ALTERNATIVE_NAME)
+            # Get the DNSName entries from the SAN extension
+            alt_names = cast(
+                list[str],
+                subject_ext.value.get_values_for_type(  # pyright: ignore
+                    DNSName))
+            self._it_subject_alt_names = '\n'.join(alt_names)
+        except ExtensionNotFound as err:
+            log.info('Certificate does not have extension: %s', err)
+            self._it_subject_alt_names = ''
 
-        serial_str = '0{:02X}'.format(cert.serial_number)
-        serial_str_foratted = ':'.join(
-            map('{}{}'.format, *(serial_str[::2], serial_str[1::2])))
-        self._it_serial_number = serial_str_foratted
+        serial_bytes = format_bytes_as_hex(
+            int_to_bytes(cert.serial_number), 2)
+        self._it_serial_number = serial_bytes
 
         self._ib_common_name = ''
         self._ib_organization = ''
@@ -109,13 +120,10 @@ class CertificateBox(Gtk.Box):
         self._expires = cert.not_valid_after.strftime('%c %Z')
 
         sha1_bytes = cert.fingerprint(hashes.SHA1())
-        sha1 = ':'.join(f'{b:02X}' for b in sha1_bytes)
-        self._sha1 = '%s\n%s' % (sha1[:29], sha1[30:])
+        self._sha1 = format_bytes_as_hex(sha1_bytes, 2)
 
         sha256_bytes = cert.fingerprint(hashes.SHA256())
-        sha256 = ':'.join(f'{b:02X}' for b in sha256_bytes)
-        self._sha256 = '%s\n%s\n%s\n%s' % (
-            sha256[:23], sha256[24:47], sha256[48:71], sha256[72:])
+        self._sha256 = format_bytes_as_hex(sha256_bytes, 4)
 
         public_key = cert.public_key()
         self._pk_algorithm = ''
@@ -132,7 +140,7 @@ class CertificateBox(Gtk.Box):
 
         self._pk_size = _('Unknown')
         if isinstance(public_key,
-                      (RSAPublicKey, DSAPublicKey, EllipticCurvePublicKey)):
+                      RSAPublicKey | DSAPublicKey | EllipticCurvePublicKey):
             self._pk_size = f'{public_key.key_size} Bit'
 
         self._ui.public_key_algorithm.set_text(self._pk_algorithm)
