@@ -12,18 +12,18 @@
 # You should have received a copy of the GNU General Public License
 # along with Gajim. If not, see <http://www.gnu.org/licenses/>.
 
-from typing import Optional
 
 import time
 from datetime import datetime
 
 from gi.repository import Gtk
 
-from gajim.common.i18n import _
 from gajim.common.const import AvatarSize
+from gajim.common.i18n import _
 
-from .widgets import SimpleLabel
-from .base import BaseRow
+from gajim.gtk.conversation.rows.base import BaseRow
+from gajim.gtk.conversation.rows.widgets import DateTimeLabel
+from gajim.gtk.conversation.rows.widgets import SimpleLabel
 
 
 class MUCJoinLeft(BaseRow):
@@ -31,9 +31,9 @@ class MUCJoinLeft(BaseRow):
                  type_: str,
                  account: str,
                  nick: str,
-                 reason: Optional[str] = None,
+                 reason: str | None = None,
                  error: bool = False,
-                 timestamp: Optional[float] = None
+                 timestamp: float | None = None
                  ) -> None:
 
         BaseRow.__init__(self, account)
@@ -65,7 +65,7 @@ class MUCJoinLeft(BaseRow):
         self._label.get_style_context().add_class('gajim-status-message')
         self.grid.attach(self._label, 2, 0, 1, 1)
 
-        timestamp_widget = self.create_timestamp_widget(self.timestamp)
+        timestamp_widget = DateTimeLabel(self.timestamp)
         timestamp_widget.set_halign(Gtk.Align.START)
         timestamp_widget.set_valign(Gtk.Align.FILL)
         self.grid.attach(timestamp_widget, 3, 0, 1, 1)
@@ -73,7 +73,7 @@ class MUCJoinLeft(BaseRow):
         self.show_all()
 
     @staticmethod
-    def _make_left_message(nick: str, reason: Optional[str],
+    def _make_left_message(nick: str, reason: str | None,
                            error: bool) -> str:
         reason = '' if reason is None else f': {reason}'
 
