@@ -16,13 +16,11 @@ from __future__ import annotations
 
 from typing import Any
 from typing import cast
-from typing import Generator
 from typing import Literal
-from typing import Optional
-from typing import Union
 
 import logging
 import time
+from collections.abc import Generator
 from datetime import datetime
 from datetime import timedelta
 
@@ -39,6 +37,7 @@ from nbxmpp.structs import MucSubject
 from gajim.common import app
 from gajim.common import events
 from gajim.common import types
+from gajim.common.const import Direction
 from gajim.common.helpers import AdditionalDataDict
 from gajim.common.helpers import get_start_of_day
 from gajim.common.helpers import to_user_string
@@ -103,13 +102,11 @@ class ConversationView(Gtk.ScrolledWindow):
         # horizontally under certain conditions (applies to GroupchatControl)
         self.get_hscrollbar().hide()
 
-        self._list_box = Gtk.ListBox()
-        self._list_box.set_selection_mode(Gtk.SelectionMode.NONE)
-        self._list_box.set_sort_func(self._sort_func)
-
-        self._contact: Optional[ChatContactT] = None
+        self._contact: ChatContactT | None = None
         self._client = None
 
+        self._list_box = Gtk.ListBox()
+
         # Keeps track of the number of rows shown in ConversationView
         self._row_count: int = 0
         self._max_row_count: int = 100
@@ -125,10 +122,10 @@ class ConversationView(Gtk.ScrolledWindow):
 
         self._current_upper: float = 0
         self._autoscroll: bool = True
-        self._request_history_at_upper: Optional[float] = None
+        self._request_history_at_upper: float | None = None
         self._upper_complete: bool = False
         self._lower_complete: bool = True
-        self._requesting: Optional[str] = None
+        self._requesting: str | None = None
         self._block_signals = False
 
         self._signal_handlers_enabled = False
@@ -148,7 +145,7 @@ class ConversationView(Gtk.ScrolledWindow):
         for row in cast(list[BaseRow], self._list_box.get_selected_rows()):
             if isinstance(row, MessageRow):
                 timestamp_formatted = row.timestamp.strftime(format_string)
-                selection_text += (f'{timestamp_formatted} - {row.name}: '
+                selection_text += (f'{timestamp_formatted} - {row.name}:\n'
                                    f'{row.get_text()}\n')
 
         clip = Gtk.Clipboard.get(Gdk.SELECTION_CLIPBOARD)
@@ -157,7 +154,7 @@ class ConversationView(Gtk.ScrolledWindow):
         self.disable_row_selection()
 
     def enable_row_selection(self,
-                             log_line_id: Optional[int]
+                             log_line_id: int | None
                              ) -> None:
 
         self._list_box.set_selection_mode(Gtk.SelectionMode.MULTIPLE)
@@ -256,6 +253,18 @@ class ConversationView(Gtk.ScrolledWindow):
             log.debug('emit %s, %s', signal_name, args)
             self.emit(signal_name, *args)
 
+    def _reset_list_box(self) -> None:
+        self._list_box.destroy()
+        self._list_box = Gtk.ListBox()
+        self._list_box.set_selection_mode(Gtk.SelectionMode.NONE)
+        self._list_box.set_sort_func(self._sort_func)
+        self._list_box.show()
+
+        current_child = self.get_child()
+        assert current_child is not None
+        current_child.destroy()
+        self.add(self._list_box)
+
     def _reset(self) -> None:
         self._current_upper = 0
         self._autoscroll = True
@@ -265,8 +274,7 @@ class ConversationView(Gtk.ScrolledWindow):
         self._requesting = None
         self.set_history_complete(True, False)
 
-        for row in self._list_box.get_children():
-            row.destroy()
+        self._reset_list_box()
 
         self._row_count = 0
         self._active_date_rows = set()
@@ -376,31 +384,30 @@ class ConversationView(Gtk.ScrolledWindow):
         assert row is not None
         return cast(BaseRow, row)
 
-    def get_first_message_row(self) -> Optional[MessageRow]:
+    def get_first_message_row(self) -> MessageRow | None:
         for row in self._list_box.get_children():
             if isinstance(row, MessageRow):
                 return row
         return None
 
-    def get_last_message_row(self) -> Optional[MessageRow]:
-        children = self._list_box.get_children()
-        children.reverse()
+    def get_last_message_row(self) -> MessageRow | None:
+        children = reversed(self._list_box.get_children())
         for row in children:
             if isinstance(row, MessageRow):
                 return row
         return None
 
-    def get_first_event_row(self) -> Optional[Union[InfoMessage, MUCJoinLeft]]:
+    def get_first_event_row(self) -> InfoMessage | MUCJoinLeft | None:
         for row in self._list_box.get_children():
-            if isinstance(row, (InfoMessage, MUCJoinLeft)):
+            if isinstance(row, InfoMessage | MUCJoinLeft):
                 return row
         return None
 
-    def get_last_event_row(self) -> Optional[Union[InfoMessage, MUCJoinLeft]]:
+    def get_last_event_row(self) -> InfoMessage | MUCJoinLeft | None:
         children = self._list_box.get_children()
         children.reverse()
         for row in children:
-            if isinstance(row, (InfoMessage, MUCJoinLeft)):
+            if isinstance(row, InfoMessage | MUCJoinLeft):
                 return row
         return None
 
@@ -412,7 +419,7 @@ class ConversationView(Gtk.ScrolledWindow):
 
     def add_muc_subject(self,
                         subject: MucSubject,
-                        timestamp: Optional[float] = None
+                        timestamp: float | None = None
                         ) -> None:
 
         muc_subject = MUCSubject(self.contact.account, subject, timestamp)
@@ -450,7 +457,7 @@ class ConversationView(Gtk.ScrolledWindow):
 
     def add_info_message(self,
                          text: str,
-                         timestamp: Optional[float] = None
+                         timestamp: float | None = None
                          ) -> None:
 
         message = InfoMessage(self.contact.account, text, timestamp)
@@ -460,13 +467,13 @@ class ConversationView(Gtk.ScrolledWindow):
         transfer_row = FileTransferRow(self.contact.account, transfer)
         self._insert_message(transfer_row)
 
-    def add_jingle_file_transfer(self,
-                                 event: Union[
-                                     events.FileRequestReceivedEvent,
-                                     events.FileRequestSent,
-                                     None] = None,
-                                 db_message: Optional[ConversationRow] = None
-                                 ) -> None:
+    def add_jingle_file_transfer(
+        self,
+        event: (events.FileRequestReceivedEvent |
+                events.FileRequestSent |
+                None) = None,
+        db_message: ConversationRow | None = None
+    ) -> None:
 
         assert isinstance(self._contact, BareContact)
         jingle_transfer_row = FileTransferJingleRow(
@@ -481,8 +488,8 @@ class ConversationView(Gtk.ScrolledWindow):
         self._insert_message(EncryptionInfoRow(event))
 
     def add_call_message(self,
-                         event: Optional[events.JingleRequestReceived] = None,
-                         db_message: Optional[ConversationRow] = None
+                         event: events.JingleRequestReceived | None = None,
+                         db_message: ConversationRow | None = None
                          ) -> None:
         assert isinstance(self._contact, BareContact)
         call_row = CallRow(
@@ -502,13 +509,13 @@ class ConversationView(Gtk.ScrolledWindow):
                     kind: str,
                     name: str,
                     timestamp: float,
-                    log_line_id: Optional[int] = None,
-                    message_id: Optional[str] = None,
-                    stanza_id: Optional[str] = None,
-                    display_marking: Optional[Displaymarking] = None,
-                    additional_data: Optional[AdditionalDataDict] = None,
-                    marker: Optional[str] = None,
-                    error: Union[CommonError, StanzaError, None] = None
+                    log_line_id: int | None = None,
+                    message_id: str | None = None,
+                    stanza_id: str | None = None,
+                    display_marking: Displaymarking | None = None,
+                    additional_data: AdditionalDataDict | None = None,
+                    marker: str | None = None,
+                    error: CommonError | StanzaError | None = None
                     ) -> None:
 
         if not timestamp:
@@ -584,7 +591,7 @@ class ConversationView(Gtk.ScrolledWindow):
             if message.is_mergeable(ancestor):
                 message.set_merged(True)
 
-    def _find_ancestor(self, message: MessageRow) -> Optional[MessageRow]:
+    def _find_ancestor(self, message: MessageRow) -> MessageRow | None:
         index = message.get_index()
         while index != 0:
             index -= 1
@@ -700,10 +707,55 @@ class ConversationView(Gtk.ScrolledWindow):
         adj = self.get_vadjustment()
         adj.set_value(adj.get_upper() - adj.get_page_size())
 
-    def _get_row_by_message_id(self, id_: str) -> Optional[MessageRow]:
+    def _get_row_by_message_id(self, id_: str) -> MessageRow | None:
         return self._message_id_row_map.get(id_)
 
-    def get_row_by_log_line_id(self, log_line_id: int) -> Optional[MessageRow]:
+    def _get_message_row_by_direction(
+        self,
+        log_line_id: int,
+        direction: Direction | None = None
+    ) -> MessageRow | None:
+
+        row = self.get_row_by_log_line_id(log_line_id)
+        if row is None:
+            return None
+
+        if direction is None:
+            return row
+
+        index = row.get_index()
+        while True:
+            if direction == Direction.PREV:
+                index -= 1
+            else:
+                index += 1
+
+            row = self._list_box.get_row_at_index(index)
+            if row is None:
+                return None
+
+            if isinstance(row, MessageRow):
+                return row
+
+    def get_prev_message_row(
+        self,
+        log_line_id: int | None
+    ) -> MessageRow | None:
+        if log_line_id is None:
+            return self.get_last_message_row()
+        return self._get_message_row_by_direction(
+            log_line_id, direction=Direction.PREV)
+
+    def get_next_message_row(
+        self,
+        log_line_id: int | None
+    ) -> MessageRow | None:
+        if log_line_id is None:
+            return None
+        return self._get_message_row_by_direction(
+            log_line_id, direction=Direction.NEXT)
+
+    def get_row_by_log_line_id(self, log_line_id: int) -> MessageRow | None:
         for row in cast(list[BaseRow], self._list_box.get_children()):
             if not isinstance(row, MessageRow):
                 continue
@@ -711,7 +763,7 @@ class ConversationView(Gtk.ScrolledWindow):
                 return row
         return None
 
-    def get_row_by_stanza_id(self, stanza_id: str) -> Optional[MessageRow]:
+    def get_row_by_stanza_id(self, stanza_id: str) -> MessageRow | None:
         for row in cast(list[BaseRow], self._list_box.get_children()):
             if not isinstance(row, MessageRow):
                 continue
@@ -753,7 +805,7 @@ class ConversationView(Gtk.ScrolledWindow):
     def correct_message(self,
                         correct_id: str,
                         text: str,
-                        nickname: Optional[str]
+                        nickname: str | None
                         ) -> None:
 
         message_row = self._get_row_by_message_id(correct_id)
@@ -780,8 +832,8 @@ class ConversationView(Gtk.ScrolledWindow):
     def _on_contact_setting_changed(self,
                                     value: Any,
                                     setting: str,
-                                    _account: Optional[str],
-                                    _jid: Optional[JID]) -> None:
+                                    _account: str | None,
+                                    _jid: JID | None) -> None:
 
         if setting == 'print_join_left':
             if value:
