@@ -18,16 +18,16 @@ from nbxmpp.protocol import JID
 from gajim.common.const import THRESHOLD_OPTIONS
 from gajim.common.i18n import _
 
-from .const import Setting
-from .const import SettingKind
-from .const import SettingType
-from .settings import SettingsBox
+from gajim.gtk.const import Setting
+from gajim.gtk.const import SettingKind
+from gajim.gtk.const import SettingType
+from gajim.gtk.settings import SettingsBox
 
 
 class GroupChatSettings(SettingsBox):
     def __init__(self, account: str, jid: JID) -> None:
         SettingsBox.__init__(self, account, str(jid))
-        self.get_style_context().add_class('settings-border')
+        self.get_style_context().add_class('border')
         self.set_selection_mode(Gtk.SelectionMode.NONE)
         self.set_size_request(700, -1)
         self.set_valign(Gtk.Align.START)
