@@ -1,3 +1,4 @@
+
 # Copyright (C) 2009-2014 Yann Leboulanger <asterix AT lagaule.org>
 #
 # This file is part of Gajim.
@@ -16,43 +17,51 @@
 
 from __future__ import annotations
 
-from typing import Iterator
 from typing import cast
-from typing import Optional
-from typing import Union
 
 import textwrap
+from collections.abc import Callable
+from collections.abc import Iterator
+from datetime import datetime
 from urllib.parse import quote
 
-from gi.repository import Gtk
 from gi.repository import Gio
 from gi.repository import GLib
-
+from gi.repository import Gtk
 from nbxmpp import JID
 
 from gajim.common import app
 from gajim.common import types
+from gajim.common.const import URIType
+from gajim.common.const import XmppUriQuery
+from gajim.common.helpers import filesystem_path_from_uri
 from gajim.common.helpers import is_affiliation_change_allowed
+from gajim.common.helpers import is_retraction_allowed
 from gajim.common.helpers import is_role_change_allowed
 from gajim.common.helpers import jid_is_blocked
 from gajim.common.i18n import _
 from gajim.common.i18n import get_short_lang_code
-from gajim.common.const import URIType
-from gajim.common.const import URIAction
-from gajim.common.structs import URI
-from gajim.common.structs import VariantMixin
-from gajim.common.modules.contacts import GroupchatContact
+from gajim.common.i18n import p_
+from gajim.common.modules.contacts import BareContact
 from gajim.common.modules.contacts import can_add_to_roster
+from gajim.common.modules.contacts import GroupchatContact
+from gajim.common.modules.contacts import GroupchatParticipant
+from gajim.common.preview import Preview
+from gajim.common.structs import URI
+from gajim.common.util.text import escape_iri_path_segment
 
-from gajim.gui.structs import AddChatActionParams
-from gajim.gui.structs import AccountJidParam
-from gajim.gui.structs import ChatListEntryParam
-from gajim.gui.structs import RemoveHistoryActionParams
-from gajim.gui.util import GajimMenu
+from gajim.gtk.const import MuteState
+from gajim.gtk.structs import AccountJidParam
+from gajim.gtk.structs import AddChatActionParams
+from gajim.gtk.structs import ChatListEntryParam
+from gajim.gtk.structs import MuteContactParam
+from gajim.gtk.structs import RemoveHistoryActionParams
+from gajim.gtk.structs import RetractMessageParam
+from gajim.gtk.util import GajimMenu
+from gajim.gtk.util import MenuItemListT
 
-
-MenuValueT = Union[None, str, GLib.Variant, VariantMixin]
-MenuItemListT = list[tuple[str, str, MenuValueT]]
+UriMenuItemsT = list[tuple[str, list[str], str]]
+UriMenuBuilderT = Callable[[URI, str], UriMenuItemsT]
 
 
 def get_self_contact_menu(contact: types.BareContact) -> GajimMenu:
@@ -61,24 +70,22 @@ def get_self_contact_menu(contact: types.BareContact) -> GajimMenu:
 
     menu = GajimMenu()
 
-    menu.add_item(_('Profile'), f'app.{account}-profile', f'"{account}"')
+    menu.add_item(_('Profile'), f'app.{account}-profile', account)
     submenu = get_send_file_submenu()
     menu.append_submenu(_('Send File'), submenu)
 
     params = RemoveHistoryActionParams(account=account, jid=jid)
     menu.add_item(_('Remove History…'), 'app.remove-history', params)
-    menu.add_item(_('Search…'), 'win.search-history', None)
     return menu
 
 
 def get_singlechat_menu(contact: types.BareContact) -> GajimMenu:
     account = contact.account
 
-    value = f'"{contact.jid}"'
-
     menu = GajimMenu()
-    menu.add_item(_('Details'), 'win.show-contact-info')
-    menu.add_item(_('Block Contact…'), f'app.{account}-block-contact', value)
+    menu.add_item(_('Block Contact…'),
+                  f'app.{account}-block-contact',
+                  str(contact.jid))
 
     submenu = get_send_file_submenu()
     menu.append_submenu(_('Send File'), submenu)
@@ -86,20 +93,25 @@ def get_singlechat_menu(contact: types.BareContact) -> GajimMenu:
     menu.add_item(_('Start Voice Call…'), 'win.start-voice-call')
     menu.add_item(_('Start Video Call…'), 'win.start-video-call')
 
-    menu.add_item(_('Search…'), 'win.search-history', None)
-
     if can_add_to_roster(contact):
         params = AccountJidParam(account=account, jid=contact.jid)
         menu.add_item(_('Add to Contact List…'), 'win.add-to-roster', params)
 
+    menu.add_item(
+        _('Execute Command…'),
+        f'app.{account}-execute-command',
+        contact.jid.bare,
+    )
+
     return menu
 
 
 def get_private_chat_menu(contact: types.GroupchatParticipant) -> GajimMenu:
     menu = GajimMenu()
-    menu.add_item(_('Details'), 'win.show-contact-info')
-    menu.add_item(_('Upload File…'), 'win.send-file-httpupload')
-    menu.add_item(_('Search…'), 'win.search-history')
+
+    value = GLib.Variant('as', [''])
+
+    menu.add_item(_('Upload File…'), 'win.send-file-httpupload', value)
 
     if can_add_to_roster(contact):
         params = AccountJidParam(account=contact.account, jid=contact.jid)
@@ -111,18 +123,18 @@ def get_private_chat_menu(contact: types.GroupchatParticipant) -> GajimMenu:
 def get_send_file_submenu() -> GajimMenu:
     menu = GajimMenu()
 
-    menu.add_item(_('Upload File…'), 'win.send-file-httpupload')
-    menu.add_item(_('Send File Directly…'), 'win.send-file-jingle')
+    value = GLib.Variant('as', [''])
+
+    menu.add_item(_('Upload File…'), 'win.send-file-httpupload', value)
+    menu.add_item(_('Send File Directly…'), 'win.send-file-jingle', value)
     return menu
 
 
 def get_groupchat_menu(contact: GroupchatContact) -> GajimMenu:
     menuitems: MenuItemListT = [
-        (_('Details'), 'win.show-contact-info', None),
         (_('Change Nickname…'), 'win.muc-change-nickname', None),
         (_('Request Voice'), 'win.muc-request-voice', None),
-        (_('Execute Command…'), 'win.muc-execute-command', '""'),
-        (_('Search…'), 'win.search-history', None)
+        (_('Execute Command…'), 'win.muc-execute-command', ''),
     ]
 
     return GajimMenu.from_list(menuitems)
@@ -130,26 +142,28 @@ def get_groupchat_menu(contact: GroupchatContact) -> GajimMenu:
 
 def get_account_menu(account: str) -> GajimMenu:
 
-    val = f'"{account}"'
+    client = app.get_client(account)
+    server_jid = client.get_own_jid().domain
 
     menuitems: MenuItemListT = [
-        (_('Profile'), f'app.{account}-profile', val),
-        (_('Discover Services…'), f'app.{account}-services', val),
-        (_('Server Info'), f'app.{account}-server-info', val),
+        (_('Profile'), f'app.{account}-profile', account),
+        (_('Discover Services…'), f'app.{account}-services', account),
+        (_('Execute Command…'), f'app.{account}-execute-command', server_jid),
+        (_('Server Info'), f'app.{account}-server-info', account),
     ]
 
     menu = GajimMenu.from_list(menuitems)
 
     advanced_menuitems: MenuItemListT = [
-        (_('Archiving Preferences'), f'app.{account}-archive', val),
-        (_('Blocking List'), f'app.{account}-blocking', val),
-        (_('PEP Configuration'), f'app.{account}-pep-config', val),
-        (_('Synchronise History…'), f'app.{account}-sync-history', val),
+        (_('Archiving Preferences'), f'app.{account}-archive', account),
+        (_('Blocking List'), f'app.{account}-blocking', account),
+        (_('PEP Configuration'), f'app.{account}-pep-config', account),
+        (_('Synchronise History…'), f'app.{account}-sync-history', account),
     ]
 
     if app.settings.get('developer_modus'):
         advanced_menuitems.append(
-            (_('Bookmarks'), f'app.{account}-bookmarks', val))
+            (_('Bookmarks'), f'app.{account}-bookmarks', account))
 
     menu.append_submenu(_('Advanced'), GajimMenu.from_list(advanced_menuitems))
 
@@ -192,16 +206,16 @@ def build_accounts_menu() -> None:
                                             'app.add-contact::')
         acc_menu.insert_item(1, add_contact_item)
         menubar.remove(menu_position)
-        menubar.insert_submenu(menu_position, _('Accounts'), acc_menu)
+        menubar.insert_submenu(menu_position, _('_Accounts'), acc_menu)
 
 
 def get_encryption_menu() -> GajimMenu:
 
     menuitems: MenuItemListT = [
-        (_('Disabled'), 'win.set-encryption', '""'),
-        ('OMEMO', 'win.set-encryption', '"OMEMO"'),
-        ('OpenPGP', 'win.set-encryption', '"OpenPGP"'),
-        ('PGP', 'win.set-encryption', '"PGP"'),
+        (_('Disabled'), 'win.set-encryption', ''),
+        ('OMEMO', 'win.set-encryption', 'OMEMO'),
+        ('OpenPGP', 'win.set-encryption', 'OpenPGP'),
+        ('PGP', 'win.set-encryption', 'PGP'),
     ]
 
     return GajimMenu.from_list(menuitems)
@@ -269,7 +283,7 @@ def get_conv_action_context_menu(account: str,
     submenu.append(item)
 
     item = Gtk.MenuItem.new_with_mnemonic(_('Open as _Link'))
-    value = GLib.Variant.new_strv([account, uri_text])
+    value = GLib.Variant.new_strv([account, f'https://{uri_text}'])
     item.set_action_target_value(value)
     submenu.append(item)
 
@@ -279,77 +293,124 @@ def get_conv_action_context_menu(account: str,
     return action_menu_item
 
 
-def get_conv_uri_context_menu(account: str, uri: URI) -> Optional[Gtk.Menu]:
-    if uri.type == URIType.XMPP:
-        if uri.action == URIAction.JOIN:
-            context_menu = [
-                ('copy-text', _('Copy XMPP Address')),
-                ('groupchat-join', _('Join Groupchat')),
-            ]
-        else:
-            context_menu = [
-                ('copy-text', _('Copy XMPP Address')),
-                ('-open-chat', _('Start Chat')),
-                ('-add-contact', _('Add to Contact List…')),
-            ]
+def _xmpp_message_query_context_menu(uri: URI, account: str) -> UriMenuItemsT:
+    jid = uri.data['jid']
+    # qparams = uri.query_params
+    return [
+        ('copy-text', [uri.source], _('Copy Link Location')),
+        ('copy-text', [jid], _('Copy XMPP Address')),
+        ('open-chat', [account, jid], _('Start Chat…')),
+        # ^ TODO: pass qparams ^
+        (f'{account}-add-contact', [account, jid], _('Add to Contact List…')),
+    ]
 
-    elif uri.type == URIType.WEB:
-        context_menu = [
-            ('copy-text', _('Copy Link Location')),
-            ('open-link', _('Open Link in Browser')),
-        ]
 
-    elif uri.type == URIType.MAIL:
-        context_menu = [
-            ('copy-text', _('Copy Email Address')),
-            ('open-mail', _('Open Email Composer')),
-        ]
+def _xmpp_join_query_context_menu(uri: URI, account: str) -> UriMenuItemsT:
+    return [
+        ('copy-text', [uri.source], _('Copy Link Location')),
+        ('copy-text', [uri.data['jid']], _('Copy XMPP Address')),
+        ('open-chat', [account, uri.data['jid']],
+            # ^ TODO: pass qparams ^
+            _('Join Groupchat…')),
+    ]
 
-    elif uri.type == URIType.GEO:
-        context_menu = [
-            ('copy-text', _('Copy Location')),
-            ('open-link', _('Show Location')),
-        ]
 
-    elif uri.type == URIType.AT:
-        context_menu = [
-            ('copy-text', _('Copy XMPP Address/Email')),
-            ('open-mail', _('Open Email Composer')),
-            ('-open-chat', _('Start Chat')),
-            ('groupchat-join', _('Join Groupchat')),
-            ('-add-contact', _('Add to Contact List…')),
-        ]
-    else:
-        return None
+_xmpp_uri_context_menus: dict[XmppUriQuery, UriMenuBuilderT] = {
+    XmppUriQuery.NONE: _xmpp_message_query_context_menu,
+    XmppUriQuery.MESSAGE: _xmpp_message_query_context_menu,
+    XmppUriQuery.JOIN: _xmpp_join_query_context_menu,
+}
 
-    menu = Gtk.Menu()
-    for item in context_menu:
-        action, label = item
+
+def _xmpp_uri_context_menu(uri: URI, account: str) -> UriMenuItemsT:
+    return _xmpp_uri_context_menus[XmppUriQuery.from_str_or_none(uri.query_type)
+                                   ](uri, account)
+
+
+def _web_uri_context_menu(uri: URI, _account: str) -> UriMenuItemsT:
+    return [
+        ('copy-text', [uri.source], _('Copy Link Location')),
+        ('open-link', ['', uri.source], _('Open Link in Browser')),
+    ]
+
+
+def _mailto_uri_context_menu(uri: URI, _account: str) -> UriMenuItemsT:
+    return [
+        ('copy-text', [uri.source], _('Copy Link Location')),
+        ('copy-text', [uri.data['addr']], _('Copy Email Address')),
+        ('open-link', ['', uri.source], _('Open Email Composer')),
+    ]
+
+
+def _geo_uri_context_menu(uri: URI, _account: str) -> UriMenuItemsT:
+    return [
+        ('copy-text', [uri.source], _('Copy Link Location')),
+        ('open-link', ['', uri.source], _('Show Location')),
+    ]
+
+
+def _ambiguous_addr_context_menu(uri: URI, account: str) -> UriMenuItemsT:
+    addr = uri.data['addr']
+    mailto = 'mailto:' + escape_iri_path_segment(addr)
+    return [
+        ('copy-text', [addr], _('Copy XMPP Address/Email')),
+        ('open-link', ['', mailto], _('Open Email Composer')),
+        ('open-chat', [account, addr], _('Start Chat…')),
+        (f'{account}-add-contact', [account, addr], _('Add to Contact List…')),
+    ]
+
+
+def _file_uri_context_menu(uri: URI, _account: str) -> UriMenuItemsT:
+    canopen = app.settings.get('allow_open_file_uris')
+    pathobj = filesystem_path_from_uri(uri.source)
+    return [
+        ('copy-text', [uri.source], _('Copy Link Location')),
+        ('open-link' if canopen else '-', ['', uri.source], _('Open Path')),
+        ('copy-text' if pathobj else '-', [str(pathobj)], _('Copy Path')),
+    ]
+
+
+def _tel_uri_context_menu(uri: URI, account: str) -> UriMenuItemsT:
+    return [
+        ('copy-text', [uri.source], _('Copy Link Location')),
+        ('open-link', [account, uri.source], _('Dial Number')),
+        # TODO:
+        # ('copy-text', [uri.data['number']], _('Copy Number')),
+    ]
+
+
+def _other_uri_context_menu(uri: URI, account: str) -> UriMenuItemsT:
+    return [
+        ('copy-text', [uri.source], _('Copy Link Location')),
+        ('open-link', [account, uri.source], _('Open Link')),
+    ]
+
+
+_uri_context_menus: dict[URIType, UriMenuBuilderT] = {
+    URIType.XMPP: _xmpp_uri_context_menu,
+    URIType.MAIL: _mailto_uri_context_menu,
+    URIType.GEO: _geo_uri_context_menu,
+    URIType.WEB: _web_uri_context_menu,
+    URIType.FILE: _file_uri_context_menu,
+    URIType.AT: _ambiguous_addr_context_menu,
+    URIType.TEL: _tel_uri_context_menu,
+    URIType.OTHER: _other_uri_context_menu,
+}
+
+
+def populate_uri_context_menu(menu: Gtk.Menu, account: str, uri: URI) -> None:
+    assert uri.type != URIType.INVALID  # it really begs to be a separate class
+    for action, args, label in _uri_context_menus[uri.type](uri, account):
         menuitem = Gtk.MenuItem()
         menuitem.set_label(label)
-
-        if action.startswith('-'):
-            action = f'app.{account}{action}'
+        menuitem.set_action_name(f'app.{action}')
+        if len(args) == 1:
+            value = GLib.Variant.new_string(args[0])
         else:
-            action = f'app.{action}'
-        menuitem.set_action_name(action)
-
-        data = uri.data
-        if uri.type == URIType.XMPP:
-            if isinstance(uri.data, dict):
-                data = uri.data['jid']
-            else:
-                data = ''
-        assert isinstance(data, str)
-
-        if action in ('app.open-mail', 'app.copy-text'):
-            value = GLib.Variant.new_string(data)
-        else:
-            value = GLib.Variant.new_strv([account, data])
+            value = GLib.Variant.new_strv(args)
         menuitem.set_action_target_value(value)
         menuitem.show()
         menu.append(menuitem)
-    return menu
 
 
 def get_roster_menu(account: str, jid: str, gateway: bool = False) -> GajimMenu:
@@ -358,7 +419,7 @@ def get_roster_menu(account: str, jid: str, gateway: bool = False) -> GajimMenu:
     if jid_is_blocked(account, jid):
         block_label = _('Unblock')
 
-    value = f'"{jid}"'
+    value = str(jid)
 
     menuitems: MenuItemListT = [
         (_('Details'), f'app.{account}-contact-info', value),
@@ -374,12 +435,31 @@ def get_roster_menu(account: str, jid: str, gateway: bool = False) -> GajimMenu:
     return GajimMenu.from_list(menuitems)
 
 
+def get_roster_view_menu() -> GajimMenu:
+    menuitems: MenuItemListT = [
+        (_('Show Offline Contacts'), 'win.show-offline', None),
+        (_('Sort by Status'), 'win.sort-by-show', None),
+        (_('Add Contact…'), 'app.add-contact', ''),
+    ]
+
+    return GajimMenu.from_list(menuitems)
+
+
+def get_account_notifications_menu(account: str) -> GajimMenu:
+    menuitems: MenuItemListT = [
+        (_('Deny all subscription requests'),
+         f'win.subscription-deny-all-{account}',
+         None),
+    ]
+    return GajimMenu.from_list(menuitems)
+
+
 def get_subscription_menu(account: str, jid: JID) -> GajimMenu:
     params = AddChatActionParams(account=account,
                                  jid=jid,
                                  type='contact',
                                  select=True)
-    value = f'"{jid}"'
+    value = str(jid)
     menuitems: MenuItemListT = [
         (_('Start Chat'), 'win.add-chat', params),
         (_('Details'), f'win.contact-info-{account}', value),
@@ -391,6 +471,19 @@ def get_subscription_menu(account: str, jid: JID) -> GajimMenu:
     return GajimMenu.from_list(menuitems)
 
 
+def get_start_chat_button_menu() -> GajimMenu:
+
+    value = GLib.Variant('as', ['', ''])
+
+    menuitems: MenuItemListT = [
+        (_('Start Chat…'), 'app.start-chat', value),
+        (_('Create Group Chat…'), 'app.create-groupchat', ''),
+        (_('Add Contact…'), 'app.add-contact', ''),
+    ]
+
+    return GajimMenu.from_list(menuitems)
+
+
 def get_start_chat_row_menu(account: str, jid: JID) -> GajimMenu:
     params = AccountJidParam(account=account, jid=jid)
     menuitems: MenuItemListT = [
@@ -408,6 +501,8 @@ def get_chat_list_row_menu(workspace_id: str,
 
     client = app.get_client(account)
     contact = client.get_module('Contacts').get_contact(jid)
+    assert isinstance(
+        contact, BareContact | GroupchatContact | GroupchatParticipant)
 
     menu = GajimMenu()
 
@@ -420,8 +515,7 @@ def get_chat_list_row_menu(workspace_id: str,
     menu.add_item(toggle_label, 'win.toggle-chat-pinned', params)
 
     submenu = menu.add_submenu(_('Move Chat'))
-    workspaces = app.settings.get_workspaces()
-    if len(workspaces) > 1:
+    if app.settings.get_workspace_count() > 1:
         for name, params in get_workspace_params(workspace_id, account, jid):
             submenu.add_item(name, 'win.move-chat-to-workspace', params)
 
@@ -441,6 +535,21 @@ def get_chat_list_row_menu(workspace_id: str,
         params = AccountJidParam(account=account, jid=jid)
         menu.add_item(_('Mark as read'), 'win.mark-as-read', params)
 
+    if contact.is_muted:
+        menu.add_item(
+            _('Unmute Chat'),
+            'app.mute-chat',
+            MuteContactParam(account=account,
+                             jid=jid,
+                             state=MuteState.UNMUTE))
+    else:
+        submenu = menu.add_submenu(_('Mute Chat'))
+        for state, label in MuteState.iter():
+            submenu.add_item(
+                label,
+                'app.mute-chat',
+                MuteContactParam(account=account, jid=jid, state=state))
+
     return menu
 
 
@@ -470,26 +579,26 @@ def get_groupchat_admin_menu(self_contact: types.GroupchatParticipant,
         return menu
 
     action = 'win.muc-change-affiliation'
+    real_jid = str(contact.real_jid)
 
     if is_affiliation_change_allowed(self_contact, contact, 'owner'):
-        value = f'["{contact.real_jid}", "owner"]'
+        value = GLib.Variant('as', [real_jid, 'owner'])
         menu.add_item(_('Make Owner'), action, value)
 
     if is_affiliation_change_allowed(self_contact, contact, 'admin'):
-        value = f'["{contact.real_jid}", "admin"]'
+        value = GLib.Variant('as', [real_jid, 'admin'])
         menu.add_item(_('Make Admin'), action, value)
 
     if is_affiliation_change_allowed(self_contact, contact, 'member'):
-        value = f'["{contact.real_jid}", "member"]'
+        value = GLib.Variant('as', [real_jid, 'member'])
         menu.add_item(_('Make Member'), action, value)
 
     if is_affiliation_change_allowed(self_contact, contact, 'none'):
-        value = f'["{contact.real_jid}", "none"]'
+        value = GLib.Variant('as', [real_jid, 'none'])
         menu.add_item(_('Revoke Member'), action, value)
 
     if is_affiliation_change_allowed(self_contact, contact, 'outcast'):
-        value = f'"{contact.real_jid}"'
-        menu.add_item(_('Ban…'), 'win.muc-ban', value)
+        menu.add_item(_('Ban…'), 'win.muc-ban', real_jid)
 
     if not menu.get_n_items():
         menu.add_item(_('Not Available'), 'dummy', None)
@@ -507,18 +616,19 @@ def get_groupchat_mod_menu(self_contact: types.GroupchatParticipant,
         menu.add_item(_('Not Available'), 'dummy', None)
         return menu
 
+    contact_name = str(contact.name)
+
     if is_role_change_allowed(self_contact, contact):
-        value = f'"{contact.name}"'
-        menu.add_item(_('Kick…'), 'win.muc-kick', value)
+        menu.add_item(_('Kick…'), 'win.muc-kick', contact_name)
 
     action = 'win.muc-change-role'
 
     if is_role_change_allowed(self_contact, contact):
         if contact.role.is_visitor:
-            value = f'["{contact.name}", "participant"]'
+            value = GLib.Variant('as', [contact_name, 'participant'])
             menu.add_item(_('Grant Voice'), action, value)
         else:
-            value = f'["{contact.name}", "visitor"]'
+            value = GLib.Variant('as', [contact_name, 'visitor'])
             menu.add_item(_('Revoke Voice'), action, value)
 
     if not menu.get_n_items():
@@ -532,7 +642,7 @@ def get_groupchat_participant_menu(account: str,
                                    contact: types.GroupchatParticipant
                                    ) -> GajimMenu:
 
-    value = f'"{contact.name}"'
+    value = str(contact.name)
 
     general_items: MenuItemListT = [
         (_('Details'), 'win.muc-contact-info', value),
@@ -541,7 +651,7 @@ def get_groupchat_participant_menu(account: str,
 
     real_contact = contact.get_real_contact()
     if real_contact is not None and can_add_to_roster(real_contact):
-        value = f'["{account}", "{real_contact.jid}"]'
+        value = GLib.Variant('as', [account, str(real_contact.jid)])
         action = f'app.{account}-add-contact'
         general_items.insert(1, (_('Add to Contact List…'), action, value))
 
@@ -554,26 +664,133 @@ def get_groupchat_participant_menu(account: str,
     return menu
 
 
-def get_component_search_menu(jid: Optional[str], copy_text: str) -> Gio.Menu:
-    menu_items: list[tuple[str, str]] = [
-        ('app.copy-text', _('Copy')),
+def get_component_search_menu(jid: str | None, copy_text: str) -> GajimMenu:
+    menuitems: MenuItemListT = [
+        (_('Copy'), 'app.copy-text', copy_text),
     ]
-    if jid is not None:
-        menu_items.append(
-            ('app.start-chat', _('Start Chat…')))
 
-    menu = Gio.Menu()
-    for item in menu_items:
-        action, label = item
-        menuitem = Gio.MenuItem.new(label, action)
-        if action == 'app.copy-text':
-            data = copy_text
-        else:
-            data = jid
-        variant = GLib.Variant('s', data)
-        menuitem.set_action_and_target_value(action, variant)
-        menu.append_item(menuitem)
-    return menu
+    if jid is not None:
+        menuitems.append(
+            (_('Start Chat…'), 'app.start-chat', jid)
+        )
+
+    return GajimMenu.from_list(menuitems)
+
+
+def get_chat_row_menu(contact: types.ChatContactT,
+                      name: str,
+                      text: str,
+                      timestamp: datetime,
+                      message_id: str | None,
+                      stanza_id: str | None,
+                      log_line_id: int | None
+                      ) -> GajimMenu:
+
+    menu_items: MenuItemListT = []
+
+    if text:
+        # Text can be an empty string,
+        # e.g. if a preview has not been loaded yet
+        timestamp_formatted = timestamp.strftime(
+            app.settings.get('date_time_format'))
+
+        copy_text = f'{timestamp_formatted} - {name}: '
+        if text.startswith(('```', '> ')):
+            # Prepend a line break in order to keep code block/quotes rendering
+            copy_text += '\n'
+        copy_text += text
+
+        menu_items.append(
+            (p_('Message row action', 'Copy'),
+            'win.copy-message',
+            copy_text))
+
+        show_quote = True
+        if isinstance(contact, GroupchatContact):
+            if contact.is_joined:
+                self_contact = contact.get_self()
+                assert self_contact is not None
+                show_quote = not self_contact.role.is_visitor
+            else:
+                show_quote = False
+        if show_quote:
+            menu_items.append((
+                p_('Message row action', 'Quote…'), 'win.quote', text))
+
+    menu_items.append(
+        (p_('Message row action', 'Select Messages…'),
+         'win.activate-message-selection',
+         GLib.Variant('u', log_line_id or 0)))
+
+    show_correction = False
+    if message_id is not None:
+        show_correction = app.window.is_message_correctable(
+            contact, message_id)
+    if show_correction:
+        menu_items.append((
+            p_('Message row action', 'Correct…'), 'win.correct-message', None))
+
+    show_retract = False
+    if isinstance(contact, GroupchatContact) and contact.is_joined:
+        resource_contact = contact.get_resource(name)
+        self_contact = contact.get_self()
+        assert self_contact is not None
+        is_allowed = is_retraction_allowed(self_contact, resource_contact)
+
+        disco_info = app.storage.cache.get_last_disco_info(contact.jid)
+        assert disco_info is not None
+
+        if disco_info.has_message_moderation and is_allowed:
+            show_retract = True
+    if show_retract and stanza_id is not None:
+        param = RetractMessageParam(
+            account=contact.account,
+            jid=contact.jid,
+            stanza_id=stanza_id)
+        menu_items.append((
+            p_('Message row action', 'Retract…'),
+            'win.retract-message',
+            param))
+
+    if log_line_id is not None:
+        menu_items.append(
+            (p_('Message row action', 'Delete Message Locally…'),
+            'win.delete-message-locally',
+            GLib.Variant('u', log_line_id or 0)))
+
+    return GajimMenu.from_list(menu_items)
+
+
+def get_preview_menu(preview: Preview) -> GajimMenu:
+    menu_items: MenuItemListT = []
+
+    download = (_('_Download'), 'win.preview-download', preview.id)
+    open_file = (_('_Open'), 'win.preview-open', preview.id)
+    save_as = (_('_Save as…'), 'win.preview-save-as', preview.id)
+    open_folder = (_('Open _Folder'), 'win.preview-open-folder', preview.id)
+    copy_link = (_('_Copy Link'), 'win.preview-copy-link', preview.id)
+    open_link = (_('Open Link in _Browser'), 'win.preview-open-link',
+                 preview.id)
+
+    if preview.is_geo_uri:
+        menu_items.append(open_file)
+        menu_items.append(copy_link)
+        return GajimMenu.from_list(menu_items)
+
+    if preview.orig_exists:
+        menu_items.append(open_file)
+        menu_items.append(save_as)
+        menu_items.append(open_folder)
+    else:
+        if not preview.download_in_progress:
+            menu_items.append(download)
+
+    menu_items.append(copy_link)
+
+    if not preview.is_aes_encrypted:
+        menu_items.append(open_link)
+
+    return GajimMenu.from_list(menu_items)
 
 
 def get_format_menu() -> GajimMenu:
@@ -589,18 +806,19 @@ def get_format_menu() -> GajimMenu:
 
 def get_workspace_menu(workspace_id: str) -> GajimMenu:
     remove_action = 'win.dummy'
-    if len(app.settings.get_workspaces()) > 1:
+    if app.settings.get_workspace_count() > 1:
         remove_action = 'win.remove-workspace'
 
     menuitems: MenuItemListT = [
-        (_('Edit…'), 'win.edit-workspace', f'"{workspace_id}"'),
-        (_('Remove'), remove_action, f'"{workspace_id}"'),
+        (_('Mark as read'), 'win.mark-workspace-as-read', workspace_id),
+        (_('Edit…'), 'win.edit-workspace', workspace_id),
+        (_('Remove'), remove_action, workspace_id),
     ]
 
     return GajimMenu.from_list(menuitems)
 
 
-def escape_mnemonic(label: Optional[str]) -> Optional[str]:
+def escape_mnemonic(label: str | None) -> str | None:
     if label is None:
         return None
     # Underscore inside a label means the next letter is a keyboard
