@@ -15,7 +15,6 @@
 from __future__ import annotations
 
 from typing import Any
-from typing import Optional
 
 import logging
 import tempfile
@@ -56,8 +55,8 @@ class MessageActionsBox(Gtk.Grid):
     def __init__(self) -> None:
         Gtk.Grid.__init__(self)
 
-        self._client: Optional[Client] = None
-        self._contact: Optional[ChatContactT] = None
+        self._client: Client | None = None
+        self._contact: ChatContactT | None = None
 
         self._ui = get_builder('message_actions_box.ui')
         self.get_style_context().add_class('message-actions-box')
@@ -103,7 +102,7 @@ class MessageActionsBox(Gtk.Grid):
         assert self._contact is not None
         return self._contact
 
-    def get_seclabel(self) -> Optional[SecurityLabel]:
+    def get_seclabel(self) -> SecurityLabel | None:
         return self._security_label_selector.get_seclabel()
 
     def _connect_actions(self) -> None:
@@ -133,7 +132,7 @@ class MessageActionsBox(Gtk.Grid):
 
     def _on_action(self,
                    action: Gio.SimpleAction,
-                   param: Optional[GLib.Variant]) -> Optional[int]:
+                   param: GLib.Variant | None) -> int | None:
 
         if self._contact is None:
             return
@@ -141,6 +140,10 @@ class MessageActionsBox(Gtk.Grid):
         action_name = action.get_name()
         log.info('Activate action: %s', action_name)
 
+        if not self.msg_textview.is_sensitive():
+            log.info('Action dismissed, input is not enabled')
+            return
+
         if action_name == 'input-clear':
             self._on_clear()
 
@@ -215,13 +218,24 @@ class MessageActionsBox(Gtk.Grid):
     def is_correcting(self) -> bool:
         return self.msg_textview.is_correcting
 
+    def insert_as_quote(self, text: str, *, clear: bool = False) -> None:
+        if self._contact is None:
+            return
+
+        if not self.msg_textview.is_sensitive():
+            return
+
+        if clear:
+            self.msg_textview.clear()
+        self.msg_textview.insert_as_quote(text)
+
     def toggle_message_correction(self) -> None:
         self.msg_textview.toggle_message_correction()
 
-    def try_message_correction(self, message: str) -> Optional[str]:
+    def try_message_correction(self, message: str) -> str | None:
         return self.msg_textview.try_message_correction(message)
 
-    def get_last_message_id(self, contact: ChatContactT) -> Optional[str]:
+    def get_last_message_id(self, contact: ChatContactT) -> str | None:
         return self.msg_textview.get_last_message_id(contact)
 
     def _on_client_state_changed(self,
@@ -428,7 +442,8 @@ class MessageActionsBox(Gtk.Grid):
                     units = GLib.FormatSizeFlags.IEC_UNITS
                 else:
                     units = GLib.FormatSizeFlags.DEFAULT
-                max_file_size = GLib.format_size_full(max_file_size, units)
+                max_file_size = GLib.format_size_full(
+                    int(max_file_size), units)
                 tooltip_text = _('Send File (max. %s)…') % max_file_size
 
         self._ui.sendfile_button.set_tooltip_text(tooltip_text)
