@@ -16,7 +16,6 @@ from __future__ import annotations
 
 from typing import Any
 from typing import cast
-from typing import Optional
 
 import logging
 import sys
@@ -71,7 +70,7 @@ class Preferences(Gtk.ApplicationWindow):
 
         self._ui = get_builder('preferences.ui')
 
-        self._video_preview: Optional[VideoPreview] = None
+        self._video_preview: VideoPreview | None = None
         self._prefs: dict[str, PreferenceBox] = {}
 
         side_bar_switcher = SideBarSwitcher()
@@ -126,13 +125,13 @@ class Preferences(Gtk.ApplicationWindow):
 
     def _add_video_preview(self) -> None:
         self._video_preview = VideoPreview()
-        self._ui.video.add(self._video_preview.widget)
+        self._ui.video.add(self._video_preview)
 
     def _on_key_press(self, _widget: Gtk.Widget, event: Gdk.EventKey) -> None:
         if event.keyval == Gdk.KEY_Escape:
             self.destroy()
 
-    def get_video_preview(self) -> Optional[VideoPreview]:
+    def get_video_preview(self) -> VideoPreview | None:
         return self._video_preview
 
     @staticmethod
@@ -420,7 +419,7 @@ class FilePreview(PreferenceBox):
                     'preview_size',
                     desc=_('Size of preview images in pixels'),
                     bind='enable_file_preview',
-                    props={'range_': (100, 1000)}),
+                    props={'range_': (100, 1000, 1)}),
 
             Setting(SettingKind.POPOVER,
                     _('File Size Limit'),
@@ -590,7 +589,7 @@ class AutoAwayDialog(SettingsDialog):
                     SettingType.CONFIG,
                     'autoawaytime',
                     desc=_('Minutes until your status gets changed'),
-                    props={'range_': (1, 720)},
+                    props={'range_': (1, 720, 1)},
                     bind='autoaway'),
 
             Setting(SettingKind.ENTRY,
@@ -618,7 +617,7 @@ class AutoExtendedAwayDialog(SettingsDialog):
                     SettingType.CONFIG,
                     'autoxatime',
                     desc=_('Minutes until your status gets changed'),
-                    props={'range_': (1, 720)},
+                    props={'range_': (1, 720, 1)},
                     bind='autoxa'),
 
             Setting(SettingKind.ENTRY,
@@ -644,6 +643,13 @@ class Themes(PreferenceBox):
         }
 
         settings = [
+            Setting(SettingKind.SPIN,
+                    _('User Interface Font Size'),
+                    SettingType.CONFIG,
+                    'app_font_size',
+                    props={'range_': (1.0, 1.5, 0.125)},
+                    callback=self._on_app_font_size_changed),
+
             Setting(SettingKind.POPOVER,
                     _('Dark Theme'),
                     SettingType.CONFIG,
@@ -664,11 +670,14 @@ class Themes(PreferenceBox):
 
         PreferenceBox.__init__(self, settings)
 
+    @staticmethod
+    def _on_app_font_size_changed(_value: float, *args: Any) -> None:
+        app.css_config.apply_app_font_size()
+
     @staticmethod
     def _get_theme_items() -> list[str]:
         theme_items = ['default']
-        for settings_theme in app.css_config.themes:
-            theme_items.append(settings_theme)
+        theme_items.extend(app.css_config.themes)
         return theme_items
 
     def update_theme_list(self) -> None:
