@@ -14,8 +14,8 @@
 # You should have received a copy of the GNU General Public License
 # along with Gajim. If not, see <http://www.gnu.org/licenses/>.
 
-import locale
 import gettext
+import locale
 from pathlib import Path
 
 from gajim.common import configpaths
@@ -29,8 +29,7 @@ except KeyError:
     plugin_user_dir = Path.cwd()
 
 
-# python 3.7 gettext module does not support Path objects
-plugins_locale_dir = str(plugin_user_dir / 'locale')
+plugins_locale_dir = plugin_user_dir / 'locale'
 
 try:
     t = gettext.translation(DOMAIN, plugins_locale_dir)
@@ -39,4 +38,4 @@ except OSError:
     _ = gettext.gettext
 
 if hasattr(locale, 'bindtextdomain'):
-    locale.bindtextdomain(DOMAIN, plugins_locale_dir)  # type: ignore
+    locale.bindtextdomain(DOMAIN, plugins_locale_dir)
