@@ -19,7 +19,6 @@
 from __future__ import annotations
 
 from typing import Any
-from typing import Optional
 from typing import TYPE_CHECKING
 
 import logging
@@ -33,24 +32,22 @@ from nbxmpp import JID
 from nbxmpp.namespaces import Namespace
 
 from gajim.common import app
-from gajim.common import configpaths
-from gajim.common import jingle_xtls
 from gajim.common import helpers
 from gajim.common.const import KindConstant
-from gajim.common.helpers import AdditionalDataDict
 from gajim.common.events import FileRequestReceivedEvent
 from gajim.common.file_props import FileProp
 from gajim.common.file_props import FilesProp
+from gajim.common.helpers import AdditionalDataDict
 from gajim.common.jingle_content import contents
 from gajim.common.jingle_content import JingleContent
+from gajim.common.jingle_ftstates import StateCandReceived
+from gajim.common.jingle_ftstates import StateCandSent
+from gajim.common.jingle_ftstates import StateCandSentAndRecv
+from gajim.common.jingle_ftstates import StateInitialized
+from gajim.common.jingle_ftstates import StateTransfering
+from gajim.common.jingle_ftstates import StateTransportReplace
 from gajim.common.jingle_transport import JingleTransportSocks5
 from gajim.common.jingle_transport import TransportType
-from gajim.common.jingle_ftstates import StateInitialized
-from gajim.common.jingle_ftstates import StateCandSent
-from gajim.common.jingle_ftstates import StateCandReceived
-from gajim.common.jingle_ftstates import StateTransfering
-from gajim.common.jingle_ftstates import StateCandSentAndRecv
-from gajim.common.jingle_ftstates import StateTransportReplace
 
 if TYPE_CHECKING:
     from gajim.common.jingle_session import JingleSession
@@ -79,9 +76,8 @@ class JingleFileTransfer(JingleContent):
     def __init__(self,
                  session: JingleSession,
                  file_props: FileProp,
-                 transport: Optional[JingleTransport] = None,
-                 use_security: bool = False,
-                 senders: Optional[str] = None
+                 transport: JingleTransport | None = None,
+                 senders: str | None = None
                  ) -> None:
 
         JingleContent.__init__(self, session, transport, senders)
@@ -102,8 +98,6 @@ class JingleFileTransfer(JingleContent):
         self.callbacks['transport-info'] += [self.__on_transport_info]
         self.callbacks['iq-result'] += [self.__on_iq_result]
 
-        self.use_security = use_security
-        self.x509_fingerprint: Optional[str] = None
         self.file_props = file_props
         self.weinitiate = self.session.weinitiate
         self.werequest = self.session.werequest
@@ -123,7 +117,7 @@ class JingleFileTransfer(JingleContent):
             self.file_props.receiver = self.session.peerjid
         self.file_props.session_type = 'jingle'
         self.file_props.sid = session.sid
-        self.file_props.transfered_size = []
+        self.file_props.transferred_size = []
         self.file_props.transport_sid = self.transport.sid
 
         self.session = session
@@ -139,15 +133,9 @@ class JingleFileTransfer(JingleContent):
             State.CAND_SENT_AND_RECEIVED: StateCandSentAndRecv(self)
         }
 
-        cert_name = (configpaths.get('MY_CERT') /
-                     jingle_xtls.SELF_SIGNED_CERTIFICATE)
-        if not (cert_name.with_suffix('.cert').exists() and
-                cert_name.with_suffix('.pkey').exists()):
-            jingle_xtls.make_certs(cert_name, 'gajim')
-
     def __state_changed(self,
                         nextstate: State,
-                        args: Optional[dict[str, Any]] = None
+                        args: dict[str, Any] | None = None
                         ) -> None:
         # Executes the next state action and sets the next state
         current_state = self.state
@@ -160,7 +148,7 @@ class JingleFileTransfer(JingleContent):
     def __on_session_initiate(self,
                               stanza: nbxmpp.Node,
                               content: nbxmpp.Node,
-                              error: Optional[nbxmpp.Node],
+                              error: nbxmpp.Node | None,
                               action: str
                               ) -> None:
         log.debug('Jingle FT request received')
@@ -200,13 +188,6 @@ class JingleFileTransfer(JingleContent):
         if not content:
             return
 
-        secu = content.getTag('security')
-        self.use_security = bool(secu)
-        if secu:
-            fingerprint = secu.getTag('fingerprint')
-            if fingerprint:
-                self.x509_fingerprint = fingerprint.getData()
-
         if not self.transport:
             self.transport = JingleTransportSocks5()
             self.transport.set_our_jid(self.session.ourjid)
@@ -262,7 +243,7 @@ class JingleFileTransfer(JingleContent):
         file_desc_tag = file_tag.getTag('desc')
         if file_desc_tag is not None:
             file_props.desc = file_desc_tag.getData()
-        file_props.transfered_size = []
+        file_props.transferred_size = []
 
         app.ged.raise_event(FileRequestReceivedEvent(
             conn=con,
@@ -276,7 +257,7 @@ class JingleFileTransfer(JingleContent):
     def __on_session_initiate_sent(self,
                                    stanza: nbxmpp.Node,
                                    content: nbxmpp.Node,
-                                   error: Optional[nbxmpp.Node],
+                                   error: nbxmpp.Node | None,
                                    action: str
                                    ) -> None:
         pass
@@ -301,13 +282,13 @@ class JingleFileTransfer(JingleContent):
         }
         self.session.connection.get_module('Jingle').set_file_info(file_info)
 
-    def _compute_hash(self) -> Optional[nbxmpp.Hashes2]:
+    def _compute_hash(self) -> nbxmpp.Hashes2 | None:
         # Calculates the hash and returns a xep-300 hash stanza
         if self.file_props.algo is None:
             return
         try:
             file_ = open(self.file_props.file_name, 'rb')
-        except IOError:
+        except OSError:
             # can't open file
             return
         h = nbxmpp.Hashes2()
@@ -329,30 +310,10 @@ class JingleFileTransfer(JingleContent):
     def __on_session_accept(self,
                             stanza: nbxmpp.Node,
                             content: nbxmpp.Node,
-                            error: Optional[nbxmpp.Node],
+                            error: nbxmpp.Node | None,
                             action: str
                             ) -> None:
         log.info('__on_session_accept')
-        con = self.session.connection
-        security = content.getTag('security')
-        if not security:  # responder can not verify our fingerprint
-            self.use_security = False
-        else:
-            fingerprint = security.getTag('fingerprint')
-            if fingerprint:
-                fingerprint = fingerprint.getData()
-                self.x509_fingerprint = fingerprint
-                if not jingle_xtls.check_cert(app.get_jid_without_resource(
-                        self.session.responder), fingerprint):
-                    id_ = jingle_xtls.send_cert_request(con,
-                                                        self.session.responder)
-                    jingle_xtls.key_exchange_pend(id_,
-                                                  self.continue_session_accept,
-                                                  [stanza])
-                    raise nbxmpp.NodeProcessed
-        self.continue_session_accept(stanza)
-
-    def continue_session_accept(self, stanza: nbxmpp.Node) -> None:
         if self.state == State.TRANSPORT_REPLACE:
             # If we are requesting we don't have the file
             if self.session.werequest:
@@ -371,16 +332,12 @@ class JingleFileTransfer(JingleContent):
             host['initiator'] = self.session.initiator
             host['target'] = self.session.responder
             host['sid'] = self.file_props.sid
-        fingerprint = None
-        if self.use_security:
-            fingerprint = 'client'
         if self.transport.type_ == TransportType.SOCKS5:
             sid = self.file_props.transport_sid
             app.socks5queue.connect_to_hosts(self.session.connection.name,
                                              sid,
                                              self.on_connect,
                                              self._on_connect_error,
-                                             fingerprint=fingerprint,
                                              receiving=False)
             raise nbxmpp.NodeProcessed
         self.__state_changed(State.TRANSFERRING)
@@ -389,7 +346,7 @@ class JingleFileTransfer(JingleContent):
     def __on_session_terminate(self,
                                stanza: nbxmpp.Node,
                                content: nbxmpp.Node,
-                               error: Optional[nbxmpp.Node],
+                               error: nbxmpp.Node | None,
                                action: str
                                ) -> None:
         log.info('__on_session_terminate')
@@ -397,7 +354,7 @@ class JingleFileTransfer(JingleContent):
     def __on_session_info(self,
                           stanza: nbxmpp.Node,
                           content: nbxmpp.Node,
-                          error: Optional[nbxmpp.Node],
+                          error: nbxmpp.Node | None,
                           action: str
                           ) -> None:
         pass
@@ -405,7 +362,7 @@ class JingleFileTransfer(JingleContent):
     def __on_transport_accept(self,
                               stanza: nbxmpp.Node,
                               content: nbxmpp.Node,
-                              error: Optional[nbxmpp.Node],
+                              error: nbxmpp.Node | None,
                               action: str
                               ) -> None:
         log.info('__on_transport_accept')
@@ -413,7 +370,7 @@ class JingleFileTransfer(JingleContent):
     def __on_transport_replace(self,
                                stanza: nbxmpp.Node,
                                content: nbxmpp.Node,
-                               error: Optional[nbxmpp.Node],
+                               error: nbxmpp.Node | None,
                                action: str
                                ) -> None:
         log.info('__on_transport_replace')
@@ -421,7 +378,7 @@ class JingleFileTransfer(JingleContent):
     def __on_transport_reject(self,
                               stanza: nbxmpp.Node,
                               content: nbxmpp.Node,
-                              error: Optional[nbxmpp.Node],
+                              error: nbxmpp.Node | None,
                               action: str
                               ) -> None:
         log.info('__on_transport_reject')
@@ -429,7 +386,7 @@ class JingleFileTransfer(JingleContent):
     def __on_transport_info(self,
                             stanza: nbxmpp.Node,
                             content: nbxmpp.Node,
-                            error: Optional[nbxmpp.Node],
+                            error: nbxmpp.Node | None,
                             action: str
                             ) -> None:
         log.info('__on_transport_info')
@@ -485,7 +442,7 @@ class JingleFileTransfer(JingleContent):
     def __on_iq_result(self,
                        stanza: nbxmpp.Node,
                        content: nbxmpp.Node,
-                       error: Optional[nbxmpp.Node],
+                       error: nbxmpp.Node | None,
                        action: str
                        ) -> None:
         log.info('__on_iq_result')
@@ -505,7 +462,7 @@ class JingleFileTransfer(JingleContent):
     def __transport_setup(self,
                           stanza: nbxmpp.Node,
                           content: nbxmpp.Node,
-                          error: Optional[nbxmpp.Node],
+                          error: nbxmpp.Node | None,
                           action: str
                           ) -> None:
         # Sets up a few transport specific things for the file transfer
@@ -550,15 +507,11 @@ class JingleFileTransfer(JingleContent):
                                        receiver)
         self.file_props.sha_str = sha_str
         port = app.settings.get('file_transfers_port')
-        fingerprint = None
-        if self.use_security:
-            fingerprint = 'server'
         listener = app.socks5queue.start_listener(
             port,
             sha_str,
             self._store_socks5_sid,
             self.file_props,
-            fingerprint=fingerprint,
             typ='sender' if self.weinitiate else 'receiver')
         if not listener:
             # send error message, notify the user
