@@ -17,12 +17,9 @@
 from __future__ import annotations
 
 from typing import Any
-from typing import Union
-from typing import Optional
 from typing import cast
 
-import functools
-
+from gi.repository import GLib
 from nbxmpp.namespaces import Namespace
 from nbxmpp.protocol import JID
 from nbxmpp.structs import BookmarkData
@@ -30,8 +27,6 @@ from nbxmpp.structs import DiscoInfo
 from nbxmpp.structs import MessageProperties
 from nbxmpp.task import Task
 
-from gi.repository import GLib
-
 from gajim.common import app
 from gajim.common import types
 from gajim.common.events import BookmarksReceived
@@ -153,7 +148,6 @@ class Bookmarks(BaseModule):
         self._compat = Namespace.BOOKMARKS_COMPAT in info.features
         self._conversion = Namespace.BOOKMARK_CONVERSION in info.features
 
-    @functools.lru_cache(maxsize=1)
     def _bookmark_module(self) -> str:
         if not self._con.get_module('PubSub').publish_options:
             return 'PrivateBookmarks'
@@ -201,7 +195,7 @@ class Bookmarks(BaseModule):
         return set_
 
     @staticmethod
-    def _convert_to_dict(bookmarks: Optional[list[BookmarkData]]
+    def _convert_to_dict(bookmarks: list[BookmarkData] | None
                          ) -> types.BookmarksDict:
         _dict: types.BookmarksDict = {}
         if not bookmarks:
@@ -211,7 +205,7 @@ class Bookmarks(BaseModule):
             _dict[bookmark.jid] = bookmark
         return _dict
 
-    def get_bookmark(self, jid: Union[str, JID]) -> Optional[BookmarkData]:
+    def get_bookmark(self, jid: str | JID) -> BookmarkData | None:
         return self._bookmarks.get(cast(JID, jid))
 
     def request_bookmarks(self) -> None:
@@ -306,13 +300,13 @@ class Bookmarks(BaseModule):
             else:
                 self.store_bookmarks(self.bookmarks)
 
-    def get_name_from_bookmark(self, jid: Union[str, JID]) -> Optional[str]:
+    def get_name_from_bookmark(self, jid: str | JID) -> str | None:
         bookmark = self._bookmarks.get(cast(JID, jid))
         if bookmark is None:
             return bookmark
         return bookmark.name
 
-    def is_bookmark(self, jid: Union[str, JID]) -> bool:
+    def is_bookmark(self, jid: str | JID) -> bool:
         return jid in self._bookmarks
 
     def _remove_timeouts(self) -> None:
@@ -320,4 +314,5 @@ class Bookmarks(BaseModule):
             GLib.source_remove(_id)
 
     def cleanup(self) -> None:
+        BaseModule.cleanup(self)
         self._remove_timeouts()
