@@ -12,9 +12,9 @@
 # You should have received a copy of the GNU General Public License
 # along with Gajim. If not, see <http://www.gnu.org/licenses/>.
 
+from typing import Any
 from typing import Literal
 from typing import TypedDict
-from typing import Union
 
 import uuid
 
@@ -126,6 +126,10 @@ IntSettings = Literal[
     'preview_size',
 ]
 
+FloatSettings = Literal[
+    'app_font_size',
+]
+
 StringSettings = Literal[
     'action_on_close',
     'additional_uri_schemes',
@@ -160,10 +164,10 @@ StringSettings = Literal[
     'video_size',
 ]
 
-AllSettings = Literal[BoolSettings, IntSettings, StringSettings]
-AllSettingsT = Union[str, int, bool, list[str]]
+AllSettings = Literal[BoolSettings, IntSettings, FloatSettings, StringSettings]
+AllSettingsT = str | int | float | bool | list[str]
 
-APP_SETTINGS = {
+APP_SETTINGS: dict[str, str | int | float | bool | list[Any]] = {
     'additional_uri_schemes': '',
     'allow_open_file_uris': False,
     'always_english_wikipedia': False,
@@ -278,6 +282,7 @@ APP_SETTINGS = {
     'use_speller': True,
     'use_stun_server': False,
     'use_urgency_hint': True,
+    'app_font_size': 1.0,
     'video_framerate': '',
     'video_input_device': 'autovideosrc',
     'video_see_self': True,
@@ -390,7 +395,7 @@ AllGroupChatSettings = Literal[BoolGroupChatSettings,
                                IntGroupChatSettings,
                                StringGroupChatSettings]
 
-AllGroupChatSettingsT = Union[str, int, bool]
+AllGroupChatSettingsT = str | int | bool
 
 
 BoolContactSettings = Literal[
@@ -407,7 +412,7 @@ StringContactSettings = Literal[
 AllContactSettings = Literal[BoolContactSettings,
                              StringContactSettings]
 
-AllContactSettingsT = Union[str, bool]
+AllContactSettingsT = str | bool
 
 
 ACCOUNT_SETTINGS = {
@@ -437,6 +442,7 @@ ACCOUNT_SETTINGS = {
         'default_workspace': '',
         'enable_gssapi': False,
         'enable_security_labels': False,
+        'encryption_default': '',
         'file_transfer_proxies': '',
         'filetransfer_preference': 'httpupload',
         'ft_send_local_ips': True,
@@ -481,7 +487,7 @@ ACCOUNT_SETTINGS = {
     },
 
     'contact': {
-        'encryption': '',
+        'encryption': HAS_ACCOUNT_DEFAULT,
         'mute_until': '',
         'send_chatstate': HAS_ACCOUNT_DEFAULT,
         'send_marker': HAS_ACCOUNT_DEFAULT,
@@ -519,7 +525,7 @@ class OpenChatSettingDetails(TypedDict):
 
 
 OpenChatsSettingT = list[OpenChatSettingDetails]
-AllWorkspaceSettingsT = Union[str, OpenChatsSettingT]
+AllWorkspaceSettingsT = str | OpenChatsSettingT
 
 
 class WorkspaceSettings(TypedDict):
