@@ -36,12 +36,11 @@
 from __future__ import annotations
 
 from typing import Any
-from typing import Callable
 from typing import cast
-from typing import Optional
 
 import os
 import sys
+from collections.abc import Callable
 from datetime import datetime
 from datetime import timedelta
 from datetime import timezone
@@ -72,6 +71,7 @@ from gajim.common.exceptions import GajimGeneralException
 from gajim.common.helpers import load_json
 from gajim.common.helpers import open_uri
 from gajim.common.i18n import _
+from gajim.common.modules.contacts import ResourceContact
 
 from gajim.gtk import menus
 from gajim.gtk import structs
@@ -226,6 +226,7 @@ class GajimApplication(Gtk.Application, CoreApplication):
 
         self._init_core()
 
+        Gtk.IconSize.register('100', 100, 100)
         icon_theme = Gtk.IconTheme.get_default()
         icon_theme.append_search_path(str(configpaths.get('ICONS')))
         load_user_iconsets()
@@ -373,7 +374,7 @@ class GajimApplication(Gtk.Application, CoreApplication):
             # Incorporate profile name into application id
             # to have a single app instance for each profile.
             profile = profile.get_string()
-            app_id = '%s.%s' % (self.get_application_id(), profile)
+            app_id = f'{self.get_application_id()}.{profile}'
             self.set_application_id(app_id)
             configpaths.set_profile(profile)
 
@@ -529,7 +530,7 @@ class GajimApplication(Gtk.Application, CoreApplication):
 
     def create_account(self,
                        account: str,
-                       username: str,
+                       username: str | None,
                        domain: str,
                        password: str,
                        proxy_name: str | None,
@@ -599,12 +600,12 @@ class GajimApplication(Gtk.Application, CoreApplication):
 
     @staticmethod
     def _on_preferences_action(_action: Gio.SimpleAction,
-                               _param: Optional[GLib.Variant]) -> None:
+                               _param: GLib.Variant | None) -> None:
         open_window('Preferences')
 
     @staticmethod
     def _on_plugins_action(_action: Gio.SimpleAction,
-                           _param: Optional[GLib.Variant]) -> None:
+                           _param: GLib.Variant | None) -> None:
         open_window('PluginsWindow')
 
     @staticmethod
@@ -623,7 +624,7 @@ class GajimApplication(Gtk.Application, CoreApplication):
 
     @staticmethod
     def _on_quit_action(_action: Gio.SimpleAction,
-                        _param: Optional[GLib.Variant]) -> None:
+                        _param: GLib.Variant | None) -> None:
         app.window.quit()
 
     @staticmethod
@@ -675,7 +676,7 @@ class GajimApplication(Gtk.Application, CoreApplication):
 
     @staticmethod
     def _on_add_account_action(_action: Gio.SimpleAction,
-                               _param: Optional[GLib.Variant]) -> None:
+                               _param: GLib.Variant | None) -> None:
         open_window('AccountWizard')
 
     @staticmethod
@@ -721,22 +722,22 @@ class GajimApplication(Gtk.Application, CoreApplication):
 
     @staticmethod
     def _on_xml_console_action(_action: Gio.SimpleAction,
-                               _param: Optional[GLib.Variant]) -> None:
+                               _param: GLib.Variant | None) -> None:
         open_window('XMLConsoleWindow')
 
     @staticmethod
     def _on_manage_proxies_action(_action: Gio.SimpleAction,
-                                  _param: Optional[GLib.Variant]) -> None:
+                                  _param: GLib.Variant | None) -> None:
         open_window('ManageProxies')
 
     @staticmethod
     def _on_content_action(_action: Gio.SimpleAction,
-                           _param: Optional[GLib.Variant]) -> None:
+                           _param: GLib.Variant | None) -> None:
         open_uri(GAJIM_WIKI_URI)
 
     @staticmethod
     def _on_join_support_chat(_action: Gio.SimpleAction,
-                              _param: Optional[GLib.Variant]) -> None:
+                              _param: GLib.Variant | None) -> None:
         accounts = app.settings.get_active_accounts()
         if len(accounts) == 1:
             app.window.show_add_join_groupchat(
@@ -746,32 +747,32 @@ class GajimApplication(Gtk.Application, CoreApplication):
 
     @staticmethod
     def _on_faq_action(_action: Gio.SimpleAction,
-                       _param: Optional[GLib.Variant]) -> None:
+                       _param: GLib.Variant | None) -> None:
         open_uri(GAJIM_FAQ_URI)
 
     @staticmethod
     def _on_privacy_policy_action(_action: Gio.SimpleAction,
-                                  _param: Optional[GLib.Variant]) -> None:
+                                  _param: GLib.Variant | None) -> None:
         open_uri(GAJIM_PRIVACY_POLICY_URI)
 
     @staticmethod
     def _on_shortcuts_action(_action: Gio.SimpleAction,
-                             _param: Optional[GLib.Variant]) -> None:
+                             _param: GLib.Variant | None) -> None:
         ShortcutsWindow()
 
     @staticmethod
     def _on_features_action(_action: Gio.SimpleAction,
-                            _param: Optional[GLib.Variant]) -> None:
+                            _param: GLib.Variant | None) -> None:
         open_window('Features')
 
     @staticmethod
     def _on_about_action(_action: Gio.SimpleAction,
-                         _param: Optional[GLib.Variant]) -> None:
+                         _param: GLib.Variant | None) -> None:
         AboutDialog()
 
     @staticmethod
     def _on_file_transfer_action(_action: Gio.SimpleAction,
-                                 _param: Optional[GLib.Variant]) -> None:
+                                 _param: GLib.Variant | None) -> None:
 
         ft = app.interface.instances['file_transfers']
         if ft.window.get_property('visible'):
@@ -836,6 +837,7 @@ class GajimApplication(Gtk.Application, CoreApplication):
 
         client = app.get_client(params.account)
         contact = client.get_module('Contacts').get_contact(params.jid)
+        assert not isinstance(contact, ResourceContact)
 
         if params.state == MuteState.UNMUTE:
             contact.settings.set('mute_until', None)
