@@ -15,7 +15,6 @@
 from __future__ import annotations
 
 from typing import Any
-from typing import Optional
 
 import pickle
 from datetime import datetime
@@ -49,7 +48,6 @@ from gajim.common.preview_helpers import guess_simple_file_type
 from gajim.common.preview_helpers import split_geo_uri
 from gajim.common.storage.draft import DraftStorage
 from gajim.common.types import ChatContactT
-from gajim.common.types import OneOnOneContactT
 
 from gajim.gtk.builder import get_builder
 from gajim.gtk.menus import get_chat_list_row_menu
@@ -92,12 +90,15 @@ class ChatListRow(Gtk.ListBoxRow):
         self._client = app.get_client(account)
 
         self.contact = self._client.get_module('Contacts').get_contact(jid)
+        assert isinstance(
+            self.contact,
+            BareContact | GroupchatContact | GroupchatParticipant)
         self._connect_contact_signals()
 
         self.contact_name: str = self.contact.name
         self.timestamp: float = 0
-        self.stanza_id: Optional[str] = None
-        self.message_id: Optional[str] = None
+        self.stanza_id: str | None = None
+        self.message_id: str | None = None
 
         self._unread_count: int = 0
         self._needs_muc_highlight: bool = False
@@ -165,7 +166,7 @@ class ChatListRow(Gtk.ListBoxRow):
                     reason = line.additional_data.get_value(
                         'retracted', 'reason')
                     message_text = get_retraction_text(
-                        self.account, retracted_by, reason)
+                        retracted_by, reason)
 
             me_nickname = None
             if line.kind in (KindConstant.CHAT_MSG_SENT,
@@ -213,7 +214,8 @@ class ChatListRow(Gtk.ListBoxRow):
 
     @property
     def unread_count(self) -> int:
-        if (self.contact.is_groupchat and not self.contact.can_notify() and
+        if (isinstance(self.contact, GroupchatContact) and
+                not self.contact.can_notify() and
                 not self._needs_muc_highlight):
             return 0
         return self._unread_count
@@ -224,14 +226,14 @@ class ChatListRow(Gtk.ListBoxRow):
         self._update_unread()
         self.emit('unread-changed')
 
-    def get_header_type(self) -> Optional[RowHeaderType]:
+    def get_header_type(self) -> RowHeaderType | None:
         header = self.get_header()
         if header is None:
             return None
         assert isinstance(header, RowHeader)
         return header.type
 
-    def set_header_type(self, header_type: Optional[RowHeaderType]) -> None:
+    def set_header_type(self, header_type: RowHeaderType | None) -> None:
         if header_type == self.get_header_type():
             return
 
@@ -256,11 +258,15 @@ class ChatListRow(Gtk.ListBoxRow):
 
     def set_message_text(self,
                          text: str,
-                         nickname: Optional[str] = None,
-                         icon_name: Optional[str] = None,
-                         additional_data: Optional[AdditionalDataDict] = None
+                         nickname: str | None = None,
+                         icon_name: str | None = None,
+                         additional_data: AdditionalDataDict | None = None
                          ) -> None:
 
+        assert isinstance(
+            self.contact,
+            BareContact | GroupchatContact | GroupchatParticipant)
+
         draft = app.storage.drafts.get(self.contact)
         if draft is not None:
             self._show_draft(draft)
@@ -324,10 +330,16 @@ class ChatListRow(Gtk.ListBoxRow):
 
     def update_avatar(self) -> None:
         scale = self.get_scale_factor()
+        assert isinstance(
+            self.contact,
+            BareContact | GroupchatContact | GroupchatParticipant)
         surface = self.contact.get_avatar(AvatarSize.ROSTER, scale)
         self._ui.avatar_image.set_from_surface(surface)
 
     def update_name(self) -> None:
+        assert isinstance(
+            self.contact,
+            BareContact | GroupchatContact | GroupchatParticipant)
         if self.type == 'pm':
             client = app.get_client(self.account)
             muc_name = get_groupchat_name(client, self.jid.new_as_bare())
@@ -345,6 +357,10 @@ class ChatListRow(Gtk.ListBoxRow):
         self._ui.timestamp_label.set_text(
             get_uf_relative_time(datetime.fromtimestamp(self.timestamp)))
 
+    def update_state(self) -> None:
+        self.update_time()
+        self._ui.mute_image.set_visible(self.contact.is_muted)
+
     def add_unread(self, text: str) -> None:
         self._unread_count += 1
         self._update_unread()
@@ -355,12 +371,12 @@ class ChatListRow(Gtk.ListBoxRow):
             self.message_id,
             self.timestamp)
 
-        if self.contact.is_groupchat:
-            needs_highlight = message_needs_highlight(
-                text,
-                self.contact.nickname,
-                self._client.get_own_jid().bare)
-            if needs_highlight:
+        if (isinstance(self.contact, GroupchatContact) and
+                self.contact.nickname is not None):
+            if message_needs_highlight(
+                    text,
+                    self.contact.nickname,
+                    self._client.get_own_jid().bare):
                 self._needs_muc_highlight = True
                 self._ui.unread_label.get_style_context().remove_class(
                     'unread-counter-silent')
@@ -376,7 +392,8 @@ class ChatListRow(Gtk.ListBoxRow):
         app.storage.cache.reset_unread_count(self.account, self.jid)
 
         # Add class again in case we were mentioned previously
-        if self.contact.is_groupchat and not self.contact.can_notify():
+        if (isinstance(self.contact, GroupchatContact) and
+                not self.contact.can_notify()):
             self._ui.unread_label.get_style_context().add_class(
                 'unread-counter-silent')
 
@@ -401,7 +418,7 @@ class ChatListRow(Gtk.ListBoxRow):
                          _draft_storage: DraftStorage,
                          _signal_name: str,
                          contact: ChatContactT,
-                         draft: str
+                         draft: str | None
                          ) -> None:
 
         if contact != self.contact:
@@ -409,7 +426,7 @@ class ChatListRow(Gtk.ListBoxRow):
 
         self._show_draft(draft)
 
-    def _show_draft(self, draft: str) -> None:
+    def _show_draft(self, draft: str | None) -> None:
         if not draft:
             self._ui.message_label.get_style_context().remove_class('draft')
             self._display_last_conversation_line()
@@ -514,25 +531,26 @@ class ChatListRow(Gtk.ListBoxRow):
         selection_data.set(drop_type, 8, byte_data)
 
     def _connect_contact_signals(self) -> None:
+        self.contact.connect('chatstate-update', self._on_chatstate_update)
         if isinstance(self.contact, BareContact):
             self.contact.connect('presence-update', self._on_presence_update)
-            self.contact.connect('chatstate-update', self._on_chatstate_update)
             self.contact.connect('nickname-update', self._on_nickname_update)
             self.contact.connect('caps-update', self._on_avatar_update)
             self.contact.connect('avatar-update', self._on_avatar_update)
 
         elif isinstance(self.contact, GroupchatContact):
             self.contact.connect('avatar-update', self._on_avatar_update)
+            self.contact.connect('disco-info-update',
+                                  self._on_disco_info_update)
             self.contact.connect('state-changed', self._on_muc_state_changed)
             self.contact.connect('mam-sync-started', self._on_mam_sync_changed)
             self.contact.connect('mam-sync-finished', self._on_mam_sync_changed)
-            self.contact.connect('mam-sync-error', self._on_mam_sync_changed)
+            self.contact.connect('mam-sync-error', self._on_mam_sync_error)
 
             self._client.connect_signal('state-changed',
                                         self._on_client_state_changed)
 
         elif isinstance(self.contact, GroupchatParticipant):
-            self.contact.connect('chatstate-update', self._on_chatstate_update)
             self.contact.connect('user-joined', self._on_muc_user_update)
             self.contact.connect('user-left', self._on_muc_user_update)
             self.contact.connect('user-avatar-update', self._on_muc_user_update)
@@ -558,6 +576,12 @@ class ChatListRow(Gtk.ListBoxRow):
                           ) -> None:
         self.update_avatar()
 
+    def _on_disco_info_update(self,
+                              _contact: ChatContactT,
+                              _signal_name: str
+                              ) -> None:
+        self.update_name()
+
     def _on_client_state_changed(self, *args: Any) -> None:
         self._update_joined_state()
 
@@ -568,6 +592,7 @@ class ChatListRow(Gtk.ListBoxRow):
         self._reset_connection_icon()
         context = self._ui.connection_icon.get_style_context()
 
+        assert isinstance(self.contact, GroupchatContact)
         if self.contact.is_joining:
             self._ui.connection_icon.set_from_icon_name(
                 'feather-refresh-cw-symbolic', Gtk.IconSize.MENU)
@@ -598,13 +623,21 @@ class ChatListRow(Gtk.ListBoxRow):
             context.add_class('info-color')
             self._ui.connection_icon.set_tooltip_text(_('Fetching messages…'))
             self._ui.connection_icon.show()
-        elif signal_name == 'mam-sync-error':
-            self._ui.connection_icon.set_from_icon_name(
-                'feather-zap-symbolic', Gtk.IconSize.MENU)
-            context.add_class('error-color')
-            self._ui.connection_icon.set_tooltip_text(
-                _('There has been an error while trying to fetch messages.'))
-            self._ui.connection_icon.show()
+
+    def _on_mam_sync_error(self,
+                           _contact: GroupchatContact,
+                           _signal_name: str,
+                           error_text: str
+                           ) -> None:
+
+        self._reset_connection_icon()
+        self._ui.connection_icon.set_from_icon_name(
+            'feather-zap-symbolic', Gtk.IconSize.MENU)
+        self._ui.connection_icon.get_style_context().add_class('error-color')
+        self._ui.connection_icon.set_tooltip_text(
+            _('There has been an error while trying to '
+              'fetch messages: %s') % error_text)
+        self._ui.connection_icon.show()
 
     def _reset_connection_icon(self) -> None:
         self._ui.connection_icon.hide()
@@ -630,9 +663,15 @@ class ChatListRow(Gtk.ListBoxRow):
         self.update_avatar()
 
     def _on_chatstate_update(self,
-                             contact: OneOnOneContactT,
+                             contact: ChatContactT,
                              _signal_name: str
                              ) -> None:
+        if contact.is_groupchat:
+            assert isinstance(contact, GroupchatContact)
+            self._ui.chatstate_image.set_visible(
+                contact.has_composing_participants())
+            return
+
         if contact.chatstate is None:
             self._ui.chatstate_image.hide()
         else:
