@@ -16,7 +16,6 @@ from __future__ import annotations
 
 from typing import Any
 from typing import Literal
-from typing import Optional
 from typing import TYPE_CHECKING
 
 import logging
@@ -76,6 +75,8 @@ class ChatPage(Gtk.Box):
         self._search_revealer.add(self._search_view)
         self._ui.right_grid.attach(self._search_revealer, 1, 0, 1, 1)
 
+        self._restore_occupants_list = False
+
         self._chat_filter = ChatFilter(icons=True)
         self._ui.filter_bar.add(self._chat_filter)
         self._ui.filter_bar_toggle.connect(
@@ -152,7 +153,16 @@ class ChatPage(Gtk.Box):
 
         self._chat_stack.show_chat(account, jid)
 
-        if self._search_revealer.get_reveal_child():
+        if (not self._search_revealer.get_reveal_child() and
+                self._restore_occupants_list and
+                self._chat_control.contact.is_groupchat):
+            # GroupchatRoster was hidden by Search initially, but Search was
+            # afterwards closed in a 1:1 chat. Only restore GroupchatRoster if
+            # a group chat was selected.
+            app.settings.set('hide_groupchat_occupants_list', False)
+            self._restore_occupants_list = False
+
+        if not app.settings.get('hide_groupchat_occupants_list'):
             self._search_view.set_context(account, jid)
 
         self.emit('chat-selected', workspace_id, account, jid)
@@ -169,14 +179,17 @@ class ChatPage(Gtk.Box):
             self._search_view.set_context(self._chat_control.contact.account,
                                           self._chat_control.contact.jid)
 
-        # Hide group chat roster in order to make some space horizontally
-        app.settings.set('hide_groupchat_occupants_list', True)
+        if not app.settings.get('hide_groupchat_occupants_list'):
+            # Hide group chat roster in order to make some space horizontally.
+            # Store state to be able to restore it when hiding search.
+            self._restore_occupants_list = True
+            app.settings.set('hide_groupchat_occupants_list', True)
 
         self._search_revealer.set_reveal_child(True)
         self._search_view.set_focus()
 
     def _on_search_hide(self, *args: Any) -> None:
-        self._search_revealer.set_reveal_child(False)
+        self.hide_search()
 
     def _on_chat_list_changed(self,
                               chat_list_stack: ChatListStack,
@@ -227,7 +240,7 @@ class ChatPage(Gtk.Box):
                                pinned: bool = False,
                                position: int = -1,
                                select: bool = False,
-                               message: Optional[str] = None) -> None:
+                               message: str | None = None) -> None:
 
         client = app.get_client(account)
 
@@ -337,6 +350,16 @@ class ChatPage(Gtk.Box):
     def hide_search(self) -> bool:
         if self._search_revealer.get_reveal_child():
             self._search_revealer.set_reveal_child(False)
+
+            if (self._restore_occupants_list and
+                    self._chat_control.has_active_chat() and
+                    self._chat_control.contact.is_groupchat):
+                # Restore GroupchatRoster only if a group chat is selected
+                # currently. If this condition isn' satisfied, it is checked
+                # again as soon as a chat is selected in _on_chat_selected.
+                app.settings.set('hide_groupchat_occupants_list', False)
+                self._restore_occupants_list = False
+
             return True
         return False
 
