@@ -14,9 +14,6 @@
 
 from __future__ import annotations
 
-from typing import Optional
-from typing import Union
-
 import logging
 import sys
 import time
@@ -67,7 +64,8 @@ class ChatStack(Gtk.Stack, EventHelper):
         self.set_vexpand(True)
         self.set_hexpand(True)
 
-        self._current_contact: Optional[ChatContactT] = None
+        self._current_contact: ChatContactT | None = None
+        self._last_quoted_id: int | None = None
 
         self.add_named(ChatPlaceholderBox(), 'empty')
 
@@ -188,11 +186,16 @@ class ChatStack(Gtk.Stack, EventHelper):
         clipboard = Gtk.Clipboard.get(Gdk.SELECTION_PRIMARY)
         old_primary_clipboard = clipboard.wait_for_text()
 
+        self._last_quoted_id = None
+
         if self._current_contact is not None:
             self._current_contact.disconnect_all_from_obj(self)
 
         client = app.get_client(account)
-        self._current_contact = client.get_module('Contacts').get_contact(jid)
+        contact = client.get_module('Contacts').get_contact(jid)
+        assert isinstance(
+            contact, BareContact | GroupchatContact | GroupchatParticipant)
+        self._current_contact = contact
 
         app.preview_manager.clear_previews()
 
@@ -225,7 +228,7 @@ class ChatStack(Gtk.Stack, EventHelper):
 
         if isinstance(self._current_contact, GroupchatContact):
             muc_data = client.get_module('MUC').get_muc_data(
-                str(self._current_contact.jid))
+                self._current_contact.jid)
             if muc_data is not None:
                 if muc_data.state.is_captcha_request:
                     self._show_chat_function_page(FunctionMode.CAPTCHA_REQUEST)
@@ -349,10 +352,10 @@ class ChatStack(Gtk.Stack, EventHelper):
 
         self._update_group_chat_actions(self._current_contact)
 
-    def _on_account_state(self,
-                          event: Union[events.AccountConnected,
-                                       events.AccountDisconnected]
-                          ) -> None:
+    def _on_account_state(
+        self,
+        event: events.AccountConnected | events.AccountDisconnected
+    ) -> None:
 
         if self._current_contact is None:
             return
@@ -410,7 +413,7 @@ class ChatStack(Gtk.Stack, EventHelper):
                 _icon, file_type = preview_helpers.guess_simple_file_type(text)
                 text = f'{file_type} ({file_name})'
 
-        sound: Optional[str] = None
+        sound: str | None = None
         msg_type = 'chat-message'
         if isinstance(contact, BareContact):
             msg_type = 'chat-message'
@@ -444,8 +447,10 @@ class ChatStack(Gtk.Stack, EventHelper):
         if tim is not None and time.mktime(time.localtime()) - tim > 1:
             sound = None
 
+        assert isinstance(
+            contact, BareContact | GroupchatContact | GroupchatParticipant)
         if app.settings.get('notification_preview_message'):
-            if text.startswith('/me') or text.startswith('/me\n'):
+            if text.startswith('/me'):
                 name = contact.name
                 if isinstance(contact, GroupchatContact):
                     name = event.properties.muc_nickname
@@ -481,6 +486,8 @@ class ChatStack(Gtk.Stack, EventHelper):
             'muc-change-role',
             'muc-change-affiliation',
             'muc-request-voice',
+            'quote-next',
+            'quote-prev',
         ]
 
         for action in actions:
@@ -514,10 +521,10 @@ class ChatStack(Gtk.Stack, EventHelper):
         online = app.account_is_connected(account)
 
         app.window.get_action('start-voice-call').set_enabled(
-            online and contact.supports_audio() and
+            online and contact.supports_audio and
             sys.platform != 'win32')
         app.window.get_action('start-video-call').set_enabled(
-            online and contact.supports_video() and
+            online and contact.supports_video and
             sys.platform != 'win32')
 
         app.window.get_action('quote').set_enabled(online)
@@ -531,9 +538,7 @@ class ChatStack(Gtk.Stack, EventHelper):
             assert self_contact
             is_visitor = self_contact.role.is_visitor
 
-        app.window.get_action('muc-change-nickname').set_enabled(
-            joined and not contact.is_irc)
-
+        app.window.get_action('muc-change-nickname').set_enabled(joined)
         app.window.get_action('muc-contact-info').set_enabled(joined)
         app.window.get_action('muc-execute-command').set_enabled(joined)
         app.window.get_action('muc-ban').set_enabled(joined)
@@ -554,11 +559,16 @@ class ChatStack(Gtk.Stack, EventHelper):
 
     def _on_action(self,
                    action: Gio.SimpleAction,
-                   param: Optional[GLib.Variant]) -> None:
+                   param: GLib.Variant | None) -> None:
+
+        if self.get_visible_child_name() != 'controls':
+            return
 
         action_name = action.get_name()
         contact = self._current_contact
-        assert contact is not None
+        if contact is None:
+            return
+
         account = contact.account
         client = app.get_client(account)
         jid = contact.jid
@@ -648,6 +658,18 @@ class ChatStack(Gtk.Stack, EventHelper):
         elif action_name == 'muc-request-voice':
             client.get_module('MUC').request_voice(contact.jid)
 
+        elif action_name.startswith('quote-'):
+            view = self._chat_control.get_conversation_view()
+            if action_name == 'quote-prev':
+                row = view.get_prev_message_row(self._last_quoted_id)
+            else:
+                row = view.get_next_message_row(self._last_quoted_id)
+
+            if row is not None:
+                self._last_quoted_id = row.log_line_id
+                self._message_action_box.insert_as_quote(
+                    row.get_text(), clear=True)
+
     def _on_drag_data_received(self,
                                _widget: Gtk.Widget,
                                _context: Gdk.DragContext,
@@ -697,8 +719,8 @@ class ChatStack(Gtk.Stack, EventHelper):
 
     def _show_chat_function_page(self,
                                  function_mode: FunctionMode,
-                                 data: Optional[str] = None,
-                                 files: Optional[list[str]] = None
+                                 data: str | None = None,
+                                 files: list[str] | None = None
                                  ) -> None:
 
         assert self._current_contact is not None
@@ -788,9 +810,10 @@ class ChatStack(Gtk.Stack, EventHelper):
         client.send_message(message_)
 
         self._message_action_box.msg_textview.clear()
+        self._last_quoted_id = None
         app.storage.drafts.set(contact, '')
 
-    def get_last_message_id(self, contact: ChatContactT) -> Optional[str]:
+    def get_last_message_id(self, contact: ChatContactT) -> str | None:
         return self._message_action_box.get_last_message_id(contact)
 
     def _close_control(self) -> None:
@@ -805,6 +828,7 @@ class ChatStack(Gtk.Stack, EventHelper):
         if self._current_contact is not None:
             self._current_contact.disconnect_all_from_obj(self)
 
+        self._last_quoted_id = None
         self.set_visible_child_name('empty')
         self._chat_banner.clear()
         self._message_action_box.clear()
@@ -825,14 +849,9 @@ class ChatPlaceholderBox(Gtk.Box):
         Gtk.Box.__init__(self, orientation=Gtk.Orientation.VERTICAL,
                          spacing=18)
         self.set_valign(Gtk.Align.CENTER)
-        pixbuf = Gtk.IconTheme.load_icon_for_scale(
-            Gtk.IconTheme.get_default(),
-            'org.gajim.Gajim-symbolic',
-            100,
-            self.get_scale_factor(),
-            Gtk.IconLookupFlags.FORCE_SIZE)
-        image = Gtk.Image.new_from_pixbuf(pixbuf)
-        image.get_style_context().add_class('dim-label')
+        image = Gtk.Image.new_from_icon_name(
+            'org.gajim.Gajim-symbolic', Gtk.IconSize.from_name('100'))
+        image.set_opacity(0.2)
         self.add(image)
 
         button = Gtk.Button(label=_('Start Chatting…'))
