@@ -14,20 +14,20 @@
 # You should have received a copy of the GNU General Public License
 # along with Gajim. If not, see <http://www.gnu.org/licenses/>.
 
+from __future__ import annotations
+
+from typing import Any
 from typing import cast
-from typing import Any
-from typing import Optional
-from typing import Union
-from typing import Type
+from typing import Literal
+from typing import overload
 
-import logging
 import itertools
+import logging
 
 from gi.repository import Gdk
 from gi.repository import Gtk
-
-from nbxmpp.simplexml import Node
 from nbxmpp.modules import dataforms
+from nbxmpp.simplexml import Node
 
 from gajim.common import app
 from gajim.common import ged
@@ -35,24 +35,24 @@ from gajim.common.events import SearchFormReceivedEvent
 from gajim.common.events import SearchResultReceivedEvent
 from gajim.common.i18n import _
 
-from .assistant import Assistant
-from .assistant import ErrorPage
-from .assistant import Page
-from .assistant import ProgressPage
-from .dataform import DataFormWidget
-from .menus import get_component_search_menu
-from .util import GajimPopover
-from .util import ensure_not_destroyed
-from .util import EventHelper
+from gajim.gtk.assistant import Assistant
+from gajim.gtk.assistant import ErrorPage
+from gajim.gtk.assistant import Page
+from gajim.gtk.assistant import ProgressPage
+from gajim.gtk.dataform import DataFormWidget
+from gajim.gtk.menus import get_component_search_menu
+from gajim.gtk.util import ensure_not_destroyed
+from gajim.gtk.util import EventHelper
+from gajim.gtk.util import GajimPopover
 
-log = logging.getLogger('gajim.gui.search')
+log = logging.getLogger('gajim.gtk.search')
 
 
 class ComponentSearch(Assistant, EventHelper):
     def __init__(self,
                  account: str,
                  jid: str,
-                 transient_for: Optional[Gtk.Window] = None
+                 transient_for: Gtk.Window | None = None
                  ) -> None:
         Assistant.__init__(self,
                            transient_for=transient_for,
@@ -76,7 +76,7 @@ class ComponentSearch(Assistant, EventHelper):
             'error': Error()
         })
 
-        progress = cast(ProgressPage, self.add_default_page('progress'))
+        progress = self.add_default_page('progress')
         progress.set_title(_('Searching'))
         progress.set_text(_('Searching…'))
 
@@ -92,13 +92,28 @@ class ComponentSearch(Assistant, EventHelper):
 
         self.show_all()
 
+    @overload
+    def get_page(self, name: Literal['prepare']) -> RequestForm: ...
+
+    @overload
+    def get_page(self, name: Literal['form']) -> SearchForm: ...
+
+    @overload
+    def get_page(self, name: Literal['result']) -> Result: ...
+
+    @overload
+    def get_page(self, name: Literal['error']) -> Error: ...
+
+    def get_page(self, name: str) -> Page:
+        return self._pages[name]
+
     def _on_button_clicked(self,
                            _assistant: Assistant,
                            button_name: str
                            ) -> None:
         if button_name == 'search':
             self.show_page('progress', Gtk.StackTransitionType.SLIDE_LEFT)
-            form = cast(SearchForm, self.get_page('form')).get_submit_form()
+            form = self.get_page('form').get_submit_form()
             self._client.get_module('Search').send_search_form(
                 self._jid, form, True)
             return
@@ -113,13 +128,12 @@ class ComponentSearch(Assistant, EventHelper):
     @ensure_not_destroyed
     def _search_form_received(self, event: SearchFormReceivedEvent) -> None:
         if not event.is_dataform:
-            error_page = cast(Error, self.get_page('error'))
-            error_page.set_text(_('Error while retrieving search form.'))
+            self.get_page('error').set_text(
+                _('Error while retrieving search form.'))
             self.show_page('error')
             return
 
-        form_page = cast(SearchForm, self.get_page('form'))
-        form_page.process_search_form(event.data)
+        self.get_page('form').process_search_form(event.data)
         self.show_page('form')
 
     @ensure_not_destroyed
@@ -127,13 +141,12 @@ class ComponentSearch(Assistant, EventHelper):
                                 event: SearchResultReceivedEvent
                                 ) -> None:
         if event.data is None:
-            error_page = cast(Error, self.get_page('error'))
-            error_page.set_text(_('Error while receiving search results.'))
+            self.get_page('error').set_text(
+                _('Error while receiving search results.'))
             self.show_page('error')
             return
 
-        result_page = cast(Result, self.get_page('result'))
-        result_page.process_result(event.data)
+        self.get_page('result').process_result(event.data)
         self.show_page('result')
 
     def _on_destroy(self, *args: Any) -> None:
@@ -171,7 +184,7 @@ class SearchForm(Page):
     def process_search_form(self, form: Node) -> None:
         self._show_form(form)
 
-    def _show_form(self, form: Optional[Node]) -> None:
+    def _show_form(self, form: Node | None) -> None:
         if self._dataform_widget is not None:
             self.remove(self._dataform_widget)
             self._dataform_widget.destroy()
@@ -210,7 +223,7 @@ class Result(Page):
         Page.__init__(self)
         self.title = _('Search Result')
 
-        self._jid_col: Optional[int] = None
+        self._jid_col: int | None = None
 
         self._label = Gtk.Label(label=_('No results found'))
         self._label.get_style_context().add_class('bold16')
@@ -219,17 +232,17 @@ class Result(Page):
 
         self._scrolled = Gtk.ScrolledWindow()
         self._scrolled.set_propagate_natural_height(True)
-        self._scrolled.get_style_context().add_class('search-scrolled')
+        self._scrolled.get_style_context().add_class('gajim-scrolled')
         self._scrolled.set_no_show_all(True)
 
         self.add(self._label)
         self.add(self._scrolled)
 
-        self._treeview: Optional[Gtk.TreeView] = None
+        self._treeview: Gtk.TreeView | None = None
 
         self.show_all()
 
-    def process_result(self, form: Optional[Node]) -> None:
+    def process_result(self, form: Node | None) -> None:
         if self._treeview is not None:
             self._scrolled.remove(self._treeview)
             self._treeview.destroy()
@@ -243,7 +256,7 @@ class Result(Page):
 
         form = dataforms.extend_form(node=form)
 
-        fieldtypes: list[Union[Type[bool], Type[str]]] = []
+        fieldtypes: list[type[bool] | type[str]] = []
         fieldvars: list[Any] = []
         index = 0
         for field in form.reported.iter_fields():
@@ -273,11 +286,12 @@ class Result(Page):
         self._treeview = Gtk.TreeView()
         self._treeview.set_hexpand(True)
         self._treeview.set_vexpand(True)
-        self._treeview.get_style_context().add_class('search-treeview')
+        self._treeview.get_style_context().add_class('gajim-treeview')
         self._treeview.connect('button-press-event', self._on_button_press)
 
         for field, counter in zip(form.reported.iter_fields(),
-                                  itertools.count()):
+                                  itertools.count(),
+                                  strict=False):
             self._treeview.append_column(
                 Gtk.TreeViewColumn(field.label,
                                    Gtk.CellRendererText(),
@@ -292,7 +306,8 @@ class Result(Page):
                          treeview: Gtk.TreeView,
                          event: Gdk.EventButton
                          ) -> bool:
-        if event.button != 3:  # Right click
+
+        if event.button != Gdk.BUTTON_SECONDARY:
             return False
 
         path = treeview.get_path_at_pos(int(event.x), int(event.y))
@@ -304,7 +319,7 @@ class Result(Page):
         assert store is not None
         assert path is not None
         iter_ = store.get_iter(path)
-        column_values = store[iter_]
+        column_values = cast(str, store[iter_])
         text = ' '.join(column_values)
 
         jid = None
