@@ -16,8 +16,6 @@ from __future__ import annotations
 
 from typing import Any
 from typing import cast
-from typing import Optional
-from typing import Union
 
 import logging
 import time
@@ -54,7 +52,7 @@ from gajim.gtk.conversation.view import ConversationView
 from gajim.gtk.groupchat_roster import GroupchatRoster
 from gajim.gtk.groupchat_state import GroupchatState
 
-HistoryRowT = Union[events.ApplicationEvent, ConversationRow]
+HistoryRowT = events.ApplicationEvent | ConversationRow
 
 REQUEST_LINES_COUNT = 20
 
@@ -123,6 +121,18 @@ class ChatControl(EventHelper):
         assert self._client is not None
         return self._client
 
+    @property
+    def is_chat(self) -> bool:
+        return isinstance(self.contact, BareContact)
+
+    @property
+    def is_privatechat(self) -> bool:
+        return isinstance(self.contact, GroupchatParticipant)
+
+    @property
+    def is_groupchat(self) -> bool:
+        return isinstance(self.contact, GroupchatContact)
+
     def is_loaded(self, account: str, jid: JID) -> bool:
         if self._contact is None:
             return False
@@ -134,6 +144,23 @@ class ChatControl(EventHelper):
     def get_group_chat_roster(self) -> GroupchatRoster:
         return self._roster
 
+    def get_conversation_view(self) -> ConversationView:
+        return self._scrolled_view
+
+    def add_command_output(self, text: str, is_error: bool) -> None:
+        self._scrolled_view.add_command_output(text, is_error)
+
+    def add_info_message(self,
+                         text: str,
+                         timestamp: float | None = None
+                         ) -> None:
+
+        self._scrolled_view.add_info_message(text, timestamp)
+
+    def drag_data_file_transfer(self, selection: Gtk.SelectionData) -> None:
+        app.window.activate_action('send-file',
+                                   GLib.Variant('as', selection.get_uris()))
+
     def clear(self) -> None:
         log.info('Clear')
 
@@ -147,9 +174,39 @@ class ChatControl(EventHelper):
         self._roster.clear()
         self.unregister_events()
 
-    def switch_contact(self, contact: Union[BareContact,
-                                            GroupchatContact,
-                                            GroupchatParticipant]) -> None:
+    def remove_message(self, log_line_id: int) -> None:
+        self._scrolled_view.remove_message(log_line_id)
+
+    def reset_view(self) -> None:
+        self._scrolled_view.reset()
+
+    def get_autoscroll(self) -> bool:
+        return self._scrolled_view.get_autoscroll()
+
+    def scroll_to_message(self, log_line_id: int, timestamp: float) -> None:
+        row = self._scrolled_view.get_row_by_log_line_id(log_line_id)
+        if row is None:
+            # Clear view and reload conversation around timestamp
+            self._scrolled_view.reset()
+            self._scrolled_view.block_signals(True)
+            before, at_after = app.storage.archive.get_conversation_around(
+                self.contact.account, self.contact.jid, timestamp)
+            self._add_messages(before)
+            self._add_messages(at_after)
+            self._scrolled_view.set_history_complete(False, False)
+
+        GLib.idle_add(self._scrolled_view.block_signals, False)
+        GLib.idle_add(
+            self._scrolled_view.scroll_to_message_and_highlight,
+            log_line_id)
+
+    def mark_as_read(self) -> None:
+        self._jump_to_end_button.reset_unread_count()
+
+    def switch_contact(
+        self,
+        contact: BareContact | GroupchatContact | GroupchatParticipant
+    ) -> None:
 
         log.info('Switch to %s (%s)', contact.jid, contact.account)
         if self._contact is not None:
@@ -198,15 +255,14 @@ class ChatControl(EventHelper):
             contact)
         if transfers is not None:
             for transfer in transfers:
-                self.add_file_transfer(transfer)
+                self._add_file_transfer(transfer)
 
         if isinstance(contact, GroupchatContact):
             if (not app.settings.get('show_subject_on_join') or
                     contact.is_joining):
                 return
 
-            muc_data = self._client.get_module('MUC').get_muc_data(
-                str(contact.jid))
+            muc_data = self._client.get_module('MUC').get_muc_data(contact.jid)
             if muc_data is not None and muc_data.subject is not None:
                 self._scrolled_view.add_muc_subject(
                     muc_data.subject, muc_data.last_subject_timestamp)
@@ -255,8 +311,9 @@ class ChatControl(EventHelper):
             return
 
         contact = self.client.get_module('Contacts').get_contact(event.fjid)
-        if isinstance(contact, BareContact):
+        if isinstance(contact, BareContact | GroupchatContact):
             return
+
         self._scrolled_view.add_user_status(self.contact.name,
                                             contact.show.value,
                                             contact.status)
@@ -280,16 +337,20 @@ class ChatControl(EventHelper):
 
         if event.correct_id:
             self._scrolled_view.correct_message(
-                event.correct_id, event.message, self.get_our_nick())
+                event.correct_id, event.message, self._get_our_nick())
             return
 
-        self.add_message(event.message,
-                         'outgoing',
-                         tim=event.timestamp,
-                         displaymarking=displaymarking,
-                         message_id=message_id,
-                         msg_log_id=event.msg_log_id,
-                         additional_data=event.additional_data)
+        name = self._get_our_nick()
+
+        self._add_message(text=event.message,
+                          kind='outgoing',
+                          name=name,
+                          timestamp=event.timestamp,
+                          displaymarking=displaymarking,
+                          msg_log_id=event.msg_log_id,
+                          message_id=message_id,
+                          stanza_id=None,
+                          additional_data=event.additional_data)
 
     def _on_message_received(self, event: events.MessageReceived) -> None:
         if not self._is_event_processable(event):
@@ -302,17 +363,20 @@ class ChatControl(EventHelper):
             return
 
         kind = 'incoming'
+        name = self.contact.name
         if event.properties.is_sent_carbon:
             kind = 'outgoing'
+            name = self._get_our_nick()
 
-        self.add_message(event.msgtxt,
-                         kind,
-                         tim=event.properties.timestamp,
-                         displaymarking=event.displaymarking,
-                         msg_log_id=event.msg_log_id,
-                         message_id=event.properties.id,
-                         stanza_id=event.stanza_id,
-                         additional_data=event.additional_data)
+        self._add_message(text=event.msgtxt,
+                          kind=kind,
+                          name=name,
+                          timestamp=event.properties.timestamp,
+                          displaymarking=event.displaymarking,
+                          msg_log_id=event.msg_log_id,
+                          message_id=event.properties.id,
+                          stanza_id=event.stanza_id,
+                          additional_data=event.additional_data)
 
     def _on_mam_message_received(self,
                                  event: events.MamMessageReceived) -> None:
@@ -326,12 +390,12 @@ class ChatControl(EventHelper):
                 return
             if event.archive_jid != self.contact.jid:
                 return
-            self.add_muc_message(event.msgtxt,
-                                 tim=event.properties.mam.timestamp,
-                                 contact=event.properties.muc_nickname,
-                                 message_id=event.properties.id,
-                                 stanza_id=event.stanza_id,
-                                 additional_data=event.additional_data)
+
+            nickname = event.properties.muc_nickname
+            if nickname == self.contact.nickname:
+                kind = 'outgoing'
+            else:
+                kind = 'incoming'
 
         else:
 
@@ -343,35 +407,42 @@ class ChatControl(EventHelper):
                     return
 
             kind = 'incoming'
+            nickname = self.contact.name
             if event.kind == KindConstant.CHAT_MSG_SENT:
                 kind = 'outgoing'
+                nickname = self._get_our_nick()
 
-            self.add_message(event.msgtxt,
-                             kind,
-                             tim=event.properties.mam.timestamp,
-                             message_id=event.properties.id,
-                             stanza_id=event.stanza_id,
-                             additional_data=event.additional_data)
+        self._add_message(text=event.msgtxt,
+                          kind=kind,
+                          name=nickname,
+                          timestamp=event.properties.mam.timestamp,
+                          displaymarking=event.displaymarking,
+                          msg_log_id=event.msg_log_id,
+                          message_id=event.properties.id,
+                          stanza_id=event.stanza_id,
+                          additional_data=event.additional_data)
 
     def _on_gc_message_received(self, event: events.GcMessageReceived) -> None:
         if not self._is_event_processable(event):
             return
 
-        if event.properties.muc_nickname is None:
-            # message from server
-            self.add_muc_message(event.msgtxt,
-                                 tim=event.properties.timestamp,
-                                 displaymarking=event.displaymarking,
-                                 additional_data=event.additional_data)
+        assert isinstance(self.contact, GroupchatContact)
+
+        nickname = event.properties.muc_nickname
+        if nickname == self.contact.nickname:
+            kind = 'outgoing'
         else:
-            self.add_muc_message(event.msgtxt,
-                                 tim=event.properties.timestamp,
-                                 contact=event.properties.muc_nickname,
-                                 displaymarking=event.displaymarking,
-                                 message_id=event.properties.id,
-                                 stanza_id=event.stanza_id,
-                                 msg_log_id=event.msg_log_id,
-                                 additional_data=event.additional_data)
+            kind = 'incoming'
+
+        self._add_message(text=event.msgtxt,
+                          kind=kind,
+                          name=nickname,
+                          timestamp=event.properties.timestamp,
+                          displaymarking=event.displaymarking,
+                          msg_log_id=event.msg_log_id,
+                          message_id=event.properties.id,
+                          stanza_id=event.stanza_id,
+                          additional_data=event.additional_data)
 
     def _on_message_updated(self, event: events.MessageUpdated) -> None:
         if not self._is_event_processable(event):
@@ -385,7 +456,6 @@ class ChatControl(EventHelper):
             return
 
         text = get_retraction_text(
-            self.contact.account,
             event.moderation.moderator_jid,
             event.moderation.reason)
         self._scrolled_view.show_message_retraction(
@@ -429,20 +499,20 @@ class ChatControl(EventHelper):
         active_jid = app.call_manager.get_active_call_jid()
         # Don't add a second row if contact upgrades to video
         if active_jid is None:
-            self.add_call_message(event=event)
+            self._add_call_message(event=event)
 
-    def _on_file_request_event(self,
-                               event: Union[events.FileRequestReceivedEvent,
-                                            events.FileRequestSent]
-                               ) -> None:
+    def _on_file_request_event(
+        self,
+        event: events.FileRequestReceivedEvent | events.FileRequestSent
+    ) -> None:
 
         if not self._is_event_processable(event):
             return
 
-        self.add_jingle_file_transfer(event=event)
+        self._add_jingle_file_transfer(event=event)
 
     def _on_http_upload_started(self, event: events.HTTPUploadStarted) -> None:
-        self.add_file_transfer(event.transfer)
+        self._add_file_transfer(event.transfer)
 
     def _on_http_upload_error(self, event: events.HTTPUploadError) -> None:
         self.add_info_message(event.error_msg)
@@ -454,21 +524,6 @@ class ChatControl(EventHelper):
         if self._allow_add_message():
             self._scrolled_view.add_encryption_info(event)
 
-    @property
-    def is_chat(self) -> bool:
-        return isinstance(self.contact, BareContact)
-
-    @property
-    def is_privatechat(self) -> bool:
-        return isinstance(self.contact, GroupchatParticipant)
-
-    @property
-    def is_groupchat(self) -> bool:
-        return isinstance(self.contact, GroupchatContact)
-
-    def mark_as_read(self) -> None:
-        self._jump_to_end_button.reset_unread_count()
-
     def _on_autoscroll_changed(self,
                                _widget: ConversationView,
                                autoscroll: bool
@@ -501,14 +556,10 @@ class ChatControl(EventHelper):
     def _on_jump_to_end(self, _button: Gtk.Button) -> None:
         self.reset_view()
 
-    def drag_data_file_transfer(self, selection: Gtk.SelectionData) -> None:
-        app.window.activate_action('send-file',
-                                   GLib.Variant('as', selection.get_uris()))
-
-    def get_our_nick(self) -> str:
+    def _get_our_nick(self) -> str:
         if isinstance(self.contact, GroupchatParticipant):
             muc_data = self.client.get_module('MUC').get_muc_data(
-                self.contact.jid.bare)
+                self.contact.jid.new_as_bare())
             if muc_data is not None:
                 return muc_data.nick
 
@@ -517,42 +568,31 @@ class ChatControl(EventHelper):
     def _allow_add_message(self) -> bool:
         return self._scrolled_view.get_lower_complete()
 
-    def add_command_output(self, text: str, is_error: bool) -> None:
-        self._scrolled_view.add_command_output(text, is_error)
-
-    def add_info_message(self,
-                         text: str,
-                         timestamp: Optional[float] = None
-                         ) -> None:
-
-        self._scrolled_view.add_info_message(text, timestamp)
-
-    def add_file_transfer(self, transfer: HTTPFileTransfer) -> None:
+    def _add_file_transfer(self, transfer: HTTPFileTransfer) -> None:
         self._scrolled_view.add_file_transfer(transfer)
 
-    def add_jingle_file_transfer(self,
-                                 event: Union[
-                                     events.FileRequestReceivedEvent,
-                                     events.FileRequestSent,
-                                     None]
-                                 ) -> None:
+    def _add_jingle_file_transfer(
+        self,
+        event: events.FileRequestReceivedEvent | events.FileRequestSent | None
+    ) -> None:
         if self._allow_add_message():
             self._scrolled_view.add_jingle_file_transfer(event)
 
-    def add_call_message(self, event: events.JingleRequestReceived) -> None:
+    def _add_call_message(self, event: events.JingleRequestReceived) -> None:
         if self._allow_add_message():
             self._scrolled_view.add_call_message(event=event)
 
     def _add_message(self,
+                     *,
                      text: str,
                      kind: str,
                      name: str,
-                     tim: float,
-                     displaymarking: Optional[Displaymarking] = None,
-                     msg_log_id: Optional[int] = None,
-                     message_id: Optional[str] = None,
-                     stanza_id: Optional[str] = None,
-                     additional_data: Optional[AdditionalDataDict] = None
+                     timestamp: float,
+                     displaymarking: Displaymarking | None,
+                     msg_log_id: int | None,
+                     message_id: str | None,
+                     stanza_id: str | None,
+                     additional_data: AdditionalDataDict | None
                      ) -> None:
 
         if additional_data is None:
@@ -563,7 +603,7 @@ class ChatControl(EventHelper):
                 text,
                 kind,
                 name,
-                tim,
+                timestamp,
                 display_marking=displaymarking,
                 message_id=message_id,
                 stanza_id=stanza_id,
@@ -578,31 +618,66 @@ class ChatControl(EventHelper):
         else:
             self._jump_to_end_button.add_unread_count()
 
-    def remove_message(self, log_line_id: int) -> None:
-        self._scrolled_view.remove_message(log_line_id)
+    def _add_messages(self, messages: list[ConversationRow]):
+        for msg in messages:
+            if msg.kind in (KindConstant.FILE_TRANSFER_INCOMING,
+                            KindConstant.FILE_TRANSFER_OUTGOING):
+                assert msg.additional_data is not None
+                if msg.additional_data.get_value('gajim', 'type') == 'jingle':
+                    self._scrolled_view.add_jingle_file_transfer(
+                        db_message=msg)
+                continue
 
-    def reset_view(self) -> None:
-        self._scrolled_view.reset()
+            if msg.kind in (KindConstant.CALL_INCOMING,
+                            KindConstant.CALL_OUTGOING):
+                self._scrolled_view.add_call_message(db_message=msg)
+                continue
 
-    def get_autoscroll(self) -> bool:
-        return self._scrolled_view.get_autoscroll()
+            if not msg.message:
+                continue
 
-    def scroll_to_message(self, log_line_id: int, timestamp: float) -> None:
-        row = self._scrolled_view.get_row_by_log_line_id(log_line_id)
-        if row is None:
-            # Clear view and reload conversation around timestamp
-            self._scrolled_view.reset()
-            self._scrolled_view.block_signals(True)
-            before, at_after = app.storage.archive.get_conversation_around(
-                self.contact.account, self.contact.jid, timestamp)
-            self.add_messages(before)
-            self.add_messages(at_after)
-            self._scrolled_view.set_history_complete(False, False)
+            message_text = msg.message
 
-        GLib.idle_add(self._scrolled_view.block_signals, False)
-        GLib.idle_add(
-            self._scrolled_view.scroll_to_message_and_highlight,
-            log_line_id)
+            contact_name = msg.contact_name
+            kind = 'incoming'
+            if msg.kind in (
+                    KindConstant.SINGLE_MSG_RECV, KindConstant.CHAT_MSG_RECV):
+                kind = 'incoming'
+                contact_name = self.contact.name
+            elif msg.kind == KindConstant.GC_MSG:
+                kind = 'incoming'
+                if contact_name is None:
+                    # Fall back to MUC name if contact name is None
+                    # (may be the case for service messages from the MUC)
+                    contact_name = self.contact.name
+            elif msg.kind in (
+                    KindConstant.SINGLE_MSG_SENT, KindConstant.CHAT_MSG_SENT):
+                kind = 'outgoing'
+                contact_name = self._get_our_nick()
+            else:
+                log.warning('kind attribute could not be processed'
+                            'while adding message')
+
+            assert contact_name is not None
+
+            if msg.additional_data is not None:
+                retracted_by = msg.additional_data.get_value('retracted', 'by')
+                if retracted_by is not None:
+                    reason = msg.additional_data.get_value(
+                        'retracted', 'reason')
+                    message_text = get_retraction_text(retracted_by, reason)
+
+            self._scrolled_view.add_message(
+                message_text,
+                kind,
+                contact_name,
+                msg.time,
+                additional_data=msg.additional_data,
+                message_id=msg.message_id,
+                stanza_id=msg.stanza_id,
+                log_line_id=msg.log_line_id,
+                marker=msg.marker,
+                error=msg.error)
 
     def _request_messages(self, before: bool) -> list[ConversationRow]:
         if before:
@@ -653,7 +728,7 @@ class ChatControl(EventHelper):
         assert self._contact is not None
         for row in rows:
             if not isinstance(row, events.ApplicationEvent):
-                self.add_messages([row])
+                self._add_messages([row])
 
             elif isinstance(row, events.MUCUserJoined):
                 self._process_muc_user_joined(row)
@@ -711,94 +786,6 @@ class ChatControl(EventHelper):
         rows.sort(key=sort_func, reverse=before)
         return rows
 
-    def add_messages(self, messages: list[ConversationRow]):
-        for msg in messages:
-            if msg.kind in (KindConstant.FILE_TRANSFER_INCOMING,
-                            KindConstant.FILE_TRANSFER_OUTGOING):
-                assert msg.additional_data is not None
-                if msg.additional_data.get_value('gajim', 'type') == 'jingle':
-                    self._scrolled_view.add_jingle_file_transfer(
-                        db_message=msg)
-                continue
-
-            if msg.kind in (KindConstant.CALL_INCOMING,
-                            KindConstant.CALL_OUTGOING):
-                self._scrolled_view.add_call_message(db_message=msg)
-                continue
-
-            if not msg.message:
-                continue
-
-            message_text = msg.message
-
-            contact_name = msg.contact_name
-            kind = 'incoming'
-            if msg.kind in (
-                    KindConstant.SINGLE_MSG_RECV, KindConstant.CHAT_MSG_RECV):
-                kind = 'incoming'
-                contact_name = self.contact.name
-            elif msg.kind == KindConstant.GC_MSG:
-                kind = 'incoming'
-                if contact_name is None:
-                    # Fall back to MUC name if contact name is None
-                    # (may be the case for service messages from the MUC)
-                    contact_name = self.contact.name
-            elif msg.kind in (
-                    KindConstant.SINGLE_MSG_SENT, KindConstant.CHAT_MSG_SENT):
-                kind = 'outgoing'
-                contact_name = self.get_our_nick()
-            else:
-                log.warning('kind attribute could not be processed'
-                            'while adding message')
-
-            assert contact_name is not None
-
-            if msg.additional_data is not None:
-                retracted_by = msg.additional_data.get_value('retracted', 'by')
-                if retracted_by is not None:
-                    reason = msg.additional_data.get_value(
-                        'retracted', 'reason')
-                    message_text = get_retraction_text(
-                        self.contact.account, retracted_by, reason)
-
-            self._scrolled_view.add_message(
-                message_text,
-                kind,
-                contact_name,
-                msg.time,
-                additional_data=msg.additional_data,
-                message_id=msg.message_id,
-                stanza_id=msg.stanza_id,
-                log_line_id=msg.log_line_id,
-                marker=msg.marker,
-                error=msg.error)
-
-    def add_message(self,
-                    text: str,
-                    kind: str,
-                    tim: float,
-                    displaymarking: Optional[Displaymarking] = None,
-                    msg_log_id: Optional[int] = None,
-                    stanza_id: Optional[str] = None,
-                    message_id: Optional[str] = None,
-                    additional_data: Optional[AdditionalDataDict] = None
-                    ) -> None:
-
-        if kind == 'incoming':
-            name = self.contact.name
-        else:
-            name = self.get_our_nick()
-
-        self._add_message(text,
-                          kind,
-                          name,
-                          tim,
-                          displaymarking=displaymarking,
-                          msg_log_id=msg_log_id,
-                          message_id=message_id,
-                          stanza_id=stanza_id,
-                          additional_data=additional_data)
-
     def _on_user_nickname_changed(self,
                                   _contact: types.GroupchatContact,
                                   _signal_name: str,
@@ -1065,7 +1052,8 @@ class ChatControl(EventHelper):
                                           event: events.MUCRoomConfigFinished
                                           ) -> None:
 
-        self.add_info_message(_('A new group chat has been created'))
+        self.add_info_message(_('A new group chat has been created'),
+                              event.timestamp)
 
     def _on_room_presence_error(self,
                                 _contact: GroupchatContact,
@@ -1079,7 +1067,7 @@ class ChatControl(EventHelper):
                                          event: events.MUCRoomPresenceError
                                          ) -> None:
 
-        self.add_info_message(_('Error: %s') % event.error)
+        self.add_info_message(_('Error: %s') % event.error, event.timestamp)
 
     def _on_room_destroyed(self,
                            _contact: GroupchatContact,
@@ -1200,35 +1188,6 @@ class ChatControl(EventHelper):
 
         self.add_info_message(message, event.timestamp)
 
-    def add_muc_message(self,
-                        text: str,
-                        tim: float,
-                        contact: str = '',
-                        displaymarking: Optional[Displaymarking] = None,
-                        message_id: Optional[str] = None,
-                        stanza_id: Optional[str] = None,
-                        msg_log_id: Optional[int] = None,
-                        additional_data: Optional[AdditionalDataDict] = None,
-                        ) -> None:
-
-        assert isinstance(self._contact, GroupchatContact)
-
-        if contact == self._contact.nickname:
-            kind = 'outgoing'
-        else:
-            kind = 'incoming'
-            # muc-specific chatstate
-
-        self._add_message(text,
-                          kind,
-                          contact,
-                          tim,
-                          displaymarking=displaymarking,
-                          message_id=message_id,
-                          stanza_id=stanza_id,
-                          msg_log_id=msg_log_id,
-                          additional_data=additional_data)
-
     def _on_room_subject(self,
                          contact: GroupchatContact,
                          _signal_name: str,
