@@ -15,7 +15,6 @@
 from __future__ import annotations
 
 from typing import Any
-from typing import Optional
 
 import locale
 import logging
@@ -107,9 +106,6 @@ class GroupchatRoster(Gtk.Revealer, EventHelper):
 
         self._ui.connect_signals(self)
 
-        app.settings.connect_signal(
-            'hide_groupchat_occupants_list', self._hide_roster)
-
         self.register_events([
             ('theme-update', ged.GUI2, self._on_theme_update),
         ])
@@ -130,6 +126,9 @@ class GroupchatRoster(Gtk.Revealer, EventHelper):
         self.set_visible(not hide_roster)
 
     def _on_reveal(self, revealer: Gtk.Revealer, param: Any) -> None:
+        if self._contact is None:
+            return
+
         if revealer.get_reveal_child():
             self._load_roster()
         else:
@@ -141,12 +140,12 @@ class GroupchatRoster(Gtk.Revealer, EventHelper):
 
         log.info('Clear')
         self._unload_roster()
+        app.settings.disconnect_signals(self)
         self._contact.disconnect_signal(self, 'state-changed')
         self._contact = None
 
     def switch_contact(self, contact: types.ChatContactT) -> None:
-        if self._contact is not None:
-            self.clear()
+        self.clear()
 
         is_groupchat = isinstance(contact, GroupchatContact)
         hide_roster = app.settings.get('hide_groupchat_occupants_list')
@@ -157,6 +156,8 @@ class GroupchatRoster(Gtk.Revealer, EventHelper):
         log.info('Switch to %s (%s)', contact.jid, contact.account)
 
         contact.connect('state-changed', self._on_muc_state_changed)
+        app.settings.connect_signal(
+            'hide_groupchat_occupants_list', self._hide_roster)
 
         self._contact = contact
 
@@ -229,7 +230,7 @@ class GroupchatRoster(Gtk.Revealer, EventHelper):
 
         return self._filter_string in model[iter_][Column.TEXT].lower()
 
-    def _get_group_iter(self, group_name: str) -> Optional[Gtk.TreeIter]:
+    def _get_group_iter(self, group_name: str) -> Gtk.TreeIter | None:
         try:
             ref = self._group_refs[group_name]
         except KeyError:
@@ -240,7 +241,7 @@ class GroupchatRoster(Gtk.Revealer, EventHelper):
             return None
         return self._store.get_iter(path)
 
-    def _get_contact_iter(self, nick: str) -> Optional[Gtk.TreeIter]:
+    def _get_contact_iter(self, nick: str) -> Gtk.TreeIter | None:
         try:
             ref = self._contact_refs[nick]
         except KeyError:
@@ -359,7 +360,7 @@ class GroupchatRoster(Gtk.Revealer, EventHelper):
                              renderer: Gtk.CellRenderer,
                              model: Gtk.TreeModel,
                              iter_: Gtk.TreeIter,
-                             _user_data: Optional[object]
+                             _user_data: object | None
                              ) -> None:
 
         has_parent = bool(model.iter_parent(iter_))
@@ -481,7 +482,7 @@ class GroupchatRoster(Gtk.Revealer, EventHelper):
                             model: Gtk.TreeModel,
                             iter1: Gtk.TreeIter,
                             iter2: Gtk.TreeIter,
-                            _user_data: Optional[object]
+                            _user_data: object | None
                             ) -> int:
         '''
         Compare two iterators to sort them
@@ -516,7 +517,7 @@ class GroupchatRoster(Gtk.Revealer, EventHelper):
         group2_index = AffiliationRoleSortOrder[group2]
         return -1 if group1_index < group2_index else 1
 
-    def enable_sort(self, enable: bool) -> None:
+    def _enable_sort(self, enable: bool) -> None:
         column = Gtk.TREE_SORTABLE_UNSORTED_SORT_COLUMN_ID
         if enable:
             column = Column.TEXT
@@ -542,7 +543,7 @@ class GroupchatRoster(Gtk.Revealer, EventHelper):
         for participant in self._contact.get_participants():
             self._add_contact(participant)
 
-        self.enable_sort(True)
+        self._enable_sort(True)
         self._roster.set_model(self._modelfilter)
 
         self._ui.search_entry.set_text('')
@@ -558,14 +559,14 @@ class GroupchatRoster(Gtk.Revealer, EventHelper):
 
         self._roster.set_model(None)
         self._store.clear()
-        self.enable_sort(False)
+        self._enable_sort(False)
 
         self._contact_refs = {}
         self._group_refs = {}
 
     def invalidate_sort(self) -> None:
-        self.enable_sort(False)
-        self.enable_sort(True)
+        self._enable_sort(False)
+        self._enable_sort(True)
 
     def _redraw(self) -> None:
         self._roster.set_model(None)
@@ -653,4 +654,6 @@ class GroupchatRoster(Gtk.Revealer, EventHelper):
         return count
 
     def _on_theme_update(self, _event: ApplicationEvent) -> None:
+        if self._contact is None:
+            return
         self._redraw()
